# SPDX-FileCopyrightText: 2024 Martin Boller
#
# SPDX-License-Identifier: GPL-3.0-or-later

from argparse import Namespace

from gvm.protocols.gmp import Gmp
from gvmtools.helper import Table


def main(gmp: Gmp, args: Namespace) -> None:
    # pylint: disable=unused-argument

    response_xml = gmp.get_alerts(filter_string="rows=-1")
    alerts_xml = response_xml.xpath("alert")

    heading = [
        "#",
        "Name",
        "Id",
        "Event",
        "Event type",
        "Method",
        "Condition",
        "In use",
    ]

    rows = []
    numberRows = 0

    print("Listing alerts.\n")

    for alert in alerts_xml:
        # Count number of reports
        numberRows = numberRows + 1
        # Cast/convert to text to show in list
        rowNumber = str(numberRows)

        name = "".join(alert.xpath("name/text()"))
        alert_id = alert.get("id")
        alert_condition = "".join(alert.xpath("condition/text()"))
        alert_method = "".join(alert.xpath("method/text()"))
        alert_event_type = "".join(alert.xpath("event/data/text()"))
        alert_event = "".join(alert.xpath("event/text()"))
        alert_inuse = "".join(alert.xpath("in_use/text()"))
        if alert_inuse == "1":
            alert_inuse = "Yes"
        else:
            alert_inuse = "No"

        rows.append(
            [
                rowNumber,
                name,
                alert_id,
                alert_event,
                alert_event_type,
                alert_method,
                alert_condition,
                alert_inuse,
            ]
        )

    print(Table(heading=heading, rows=rows))


if __name__ == "__gmp__":
    main(gmp, args)



# SPDX-FileCopyrightText: 2017-2021 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

# pylint: disable=too-many-lines

import logging
import os
import re
import signal
import sqlite3
import sys
import tempfile
from argparse import ArgumentParser, Namespace, RawTextHelpFormatter
from datetime import datetime, timedelta, tzinfo
from decimal import Decimal
from pathlib import Path

from gvm.protocols.gmp import Gmp
from lxml import etree

__version__ = "21.7.0"

logger = logging.getLogger(__name__)

HELP_TEXT = f"""
    Check-GMP Nagios Command Plugin {__version__} (C) 2017-2021 Greenbone AG

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    """

NAGIOS_OK = 0
NAGIOS_WARNING = 1
NAGIOS_CRITICAL = 2
NAGIOS_UNKNOWN = 3

NAGIOS_MSG = ["OK", "WARNING", "CRITICAL", "UNKNOWN"]

MAX_RUNNING_INSTANCES = 10


class InstanceManager:
    """Class for managing instances of this plugin

    All new reports will be cached in a sqlite database.
    The first call with a unknown host takes longer,
    because the remote gvmd/openvasmd has to generate the report.
    The second call will retrieve the data from the database if the scan
    duration does not differ.

    Additionally this class handles all instances of check-gmp. No more than
    MAX_RUNNING_INSTANCES can run simultaneously. Other instances are stopped
    and wait for continuation.
    """

    def __init__(self, path, parser):
        """Initialise the sqlite database.

        Create it if it does not exist else connect to it.

        Arguments:
            path (string): Path to the database.
        """
        self.cursor = None
        self.con_db = None
        self.db = Path(path)
        self.pid = os.getpid()

        # Try to read file with information about cached reports
        # First check whether the file exist or not
        try:
            exist = self.db.is_file()
            logger.debug("DB file exist?: %s ", exist)

            if not exist:
                if not self.db.parent.is_dir():
                    self.db.parent.mkdir(parents=True, exist_ok=True)
                else:
                    self.db.touch()
                # Connect to db
                self.connect_db()

                # Create the tables
                self.cursor.execute(
                    """CREATE TABLE Report(
                    host text,
                    scan_end text,
                    params_used text,
                    report text
                )"""
                )

                self.cursor.execute(
                    """CREATE TABLE Instance(
                    created_at text,
                    pid integer,
                    pending integer default 0
                )"""
                )

                logger.debug("Tables created")
            else:
                self.connect_db()

        except PermissionError:
            parser.error(
                f"The selected temporary database file {self.db} or the parent "
                "dir has not the correct permissions."
            )

    @staticmethod
    def _to_sql_bool(pending):
        """Replace True/False with 1/0."""
        return "1" if pending else "0"

    def connect_db(self):
        """Connect to the database

        Simply connect to the database at location <path>
        """
        try:
            logger.debug("connect db: %s", self.db)
            self.con_db = sqlite3.connect(str(self.db))
            self.cursor = self.con_db.cursor()
            logger.debug(sqlite3.sqlite_version)
        except Exception as e:  # pylint: disable=broad-except
            logger.debug(e)

    def close_db(self):
        """Close database"""
        self.con_db.close()

    def set_host(self, host):
        """Sets the host variable

        Arguments:
            host (string): Given ip or hostname of target.
        """
        self.host = host

    def is_old_report(self, last_scan_end, params_used):
        """Decide whether the current report is old or not

        At first the last scanend and the params that were used are fetched
        from the database. If no report is fetched, then True will be returned.
        The next step is to compare the old and the new scanend.
        If the scanends matches, then return False, because it is the same
        report. Else the old report will be deleted.

        Arguments:
            last_scan_end (string): Last scan end of report
            params_used (string): Params used for this check

        Returns:
            True if it is an old report or empty. False if it is the same
            report.
        """

        # Before we do anything here, check existing instance

        # Retrieve the scan_end value
        self.cursor.execute(
            "SELECT scan_end, params_used FROM Report WHERE host=?",
            (self.host,),
        )
        db_entry = self.cursor.fetchone()

        logger.debug("%s %s", db_entry, last_scan_end)

        if not db_entry:
            return True
        else:
            old = parse_date(db_entry[0])
            new = parse_date(last_scan_end)

            logger.debug(
                "Old time (from db): %s\nNew time (from rp): %s", old, new
            )

            if new <= old and params_used == db_entry[1]:
                return False
            else:
                # Report is newer. Delete old entry.
                logger.debug("Delete old report for host %s", self.host)
                self.delete_report()
                return True

    def load_local_report(self):
        """Load report from local database

        Select the report from the database according due the hostname or ip.

        Returns:
            An lxml ElementTree
        """
        self.cursor.execute(
            "SELECT report FROM Report WHERE host=?", (self.host,)
        )
        db_entry = self.cursor.fetchone()

        if db_entry:
            return etree.fromstring(db_entry[0])
        else:
            logger.debug("Report from host %s is not in the db", self.host)

    def add_report(self, scan_end, params_used, report):
        """Create new entry with the lxml report

        Create a string from the lxml object and add it to the database.
        Additional data is the scanend and the params used.

        Arguments:
            scan_end (string): Scan end of the report
            params_used (string): Params used for this check
            report (obj): An lxml ElementTree
        """

        data = etree.tostring(report)

        logger.debug("add_report: %s, %s, %s", self.host, scan_end, params_used)

        # Insert values
        self.cursor.execute(
            "INSERT INTO Report VALUES (?, ?, ?, ?)",
            (self.host, scan_end, params_used, data),
        )

        # Save the changes
        self.con_db.commit()

    def delete_report(self):
        """Delete report from database"""
        self.cursor.execute("DELETE FROM Report WHERE host=?", (self.host,))

        # Save the changes
        self.con_db.commit()

    def delete_entry_with_ip(self, ip):
        """Delete report from database with given ip

        Arguments:
            ip (string): IP-Adress
        """
        logger.debug("Delete entry with ip: %s", ip)
        self.cursor.execute("DELETE FROM Report WHERE host=?", (ip,))
        self.con_db.isolation_level = None
        self.cursor.execute("VACUUM")
        self.con_db.isolation_level = ""  # see: https://github.com/CxAalto/gtfspy/commit/8d05c3c94a6d4ca3ed675d88af93def7d5053bfe # pylint: disable=line-too-long # noqa: E501
        # Save the changes
        self.con_db.commit()

    def delete_older_entries(self, days):
        """Delete reports from database older than given days

        Arguments:
            days (int): Number of days in past
        """
        logger.debug("Delete entries older than: %s days", days)
        self.cursor.execute(
            "DELETE FROM Report WHERE scan_end <= "
            f'date("now", "-{days} day")'
        )
        self.cursor.execute("VACUUM")

        # Save the changes
        self.con_db.commit()

    def has_entries(self, pending):
        """Return number of instance entries
        Arguments:
            pending (bool): True for pending instances. False for running
                           instances.

        Returns:
            The number of pending or non pending instances entries.
        """
        self.cursor.execute(
            "SELECT count(*) FROM Instance WHERE pending=?",
            (self._to_sql_bool(pending),),
        )

        res = self.cursor.fetchone()

        return res[0]

    def check_instances(self):
        """This method checks the status of check-gmp instances.

        Checks whether instances are pending or not and start instances
        according to the number saved in the MAX_RUNNING_INSTANCES variable.
        """

        # Need to check whether any instances are in the database that were
        # killed f.e. because a restart of nagios
        self.clean_orphaned_instances()

        # How many processes are currently running?
        number_instances = self.has_entries(pending=False)

        # How many pending entries are waiting?
        number_pending_instances = self.has_entries(pending=True)

        logger.debug(
            "check_instances: %i %i", number_instances, number_pending_instances
        )

        if (
            number_instances < MAX_RUNNING_INSTANCES
            and number_pending_instances == 0
        ):
            # Add entry for running process and go on
            logger.debug("Fall 1")
            self.add_instance(pending=False)

        elif (
            number_instances < MAX_RUNNING_INSTANCES
            and number_pending_instances > 0
        ):
            # Change pending entries and wake them up until enough instances
            # are running
            logger.debug("Fall 2")

            while (
                number_instances < MAX_RUNNING_INSTANCES
                and number_pending_instances > 0
            ):
                pending_entries = self.get_oldest_pending_entries(
                    MAX_RUNNING_INSTANCES - number_instances
                )

                logger.debug("Oldest pending pids: %s", pending_entries)

                for entry in pending_entries:
                    created_at = entry[0]
                    pid = entry[1]

                    # Change status to not pending and continue the process
                    self.update_pending_status(created_at, False)
                    self.start_process(pid)

                # Refresh number of instances for next while loop
                number_instances = self.has_entries(pending=False)
                number_pending_instances = self.has_entries(pending=True)

            # TODO: Check if this is really necessary
            # self.add_instance(pending=False)
            # if number_instances >= MAX_RUNNING_INSTANCES:
            # self.stop_process(self.pid)

        elif (
            number_instances >= MAX_RUNNING_INSTANCES
            and number_pending_instances == 0
        ):
            # There are running enough instances and no pending instances
            # Add new entry with pending status true and stop this instance
            logger.debug("Fall 3")
            self.add_instance(pending=True)
            self.stop_process(self.pid)

        elif (
            number_instances >= MAX_RUNNING_INSTANCES
            and number_pending_instances > 0
        ):
            # There are running enough instances and there are min one
            # pending instance
            # Add new entry with pending true and stop this instance
            logger.debug("Fall 4")
            self.add_instance(pending=True)
            self.stop_process(self.pid)

        # If an entry is pending and the same params at another process is
        # starting, then exit with gmp pending since data
        # if self.has_pending_entries():
        # Check if an pending entry is the same as this process
        # If hostname
        #    date = datetime.now()
        #    end_session('GMP PENDING: since %s' % date, NAGIOS_OK)
        #    end_session('GMP RUNNING: since', NAGIOS_OK)

    def add_instance(self, pending):
        """Add new instance entry to database

        Retrieve the current time in ISO 8601 format. Create a new entry with
        pending status and the dedicated pid

        Arguments:
            pending (bool): State of instance
        """
        current_time = datetime.now().isoformat()

        # Insert values
        self.cursor.execute(
            "INSERT INTO Instance VALUES (?, ?, ?)",
            (current_time, self.pid, self._to_sql_bool(pending)),
        )

        # Save the changes
        self.con_db.commit()

    def get_oldest_pending_entries(self, number):
        """Return the oldest last entries of pending entries from database

        Return:
            the oldest instances with status pending limited by the variable
            <number>
        """
        self.cursor.execute(
            "SELECT * FROM Instance WHERE pending=1 ORDER BY "
            "created_at LIMIT ? ",
            (number,),
        )
        return self.cursor.fetchall()

    def update_pending_status(self, date, pending):
        """Update pending status of instance

        The date variable works as a primary key for the instance table.
        The entry with date get his pending status updated.

        Arguments:
            date (string):  Date of creation for entry
            pending (bool): Status of instance
        """
        self.cursor.execute(
            "UPDATE Instance SET pending=? WHERE created_at=?",
            (self._to_sql_bool(pending), date),
        )

        # Save the changes
        self.con_db.commit()

    def delete_instance(self, pid=None):
        """Delete instance from database

        If a pid different from zero is given, then delete the entry with
        given pid. Else delete the entry with the pid stored in this class
        instance.

        Keyword Arguments:
            pid (number): Process Indentificattion Number (default: {0})
        """
        if not pid:
            pid = self.pid

        logger.debug("Delete entry with pid: %i", pid)
        self.cursor.execute("DELETE FROM Instance WHERE pid=?", (pid,))

        # Save the changes
        self.con_db.commit()

    def clean_orphaned_instances(self):
        """Delete non existing instance entries

        This method check whether a pid exist on the os and if not then delete
        the orphaned entry from database.
        """
        self.cursor.execute("SELECT pid FROM Instance")

        pids = self.cursor.fetchall()

        for pid in pids:
            if not self.check_pid(pid[0]):
                self.delete_instance(pid[0])

    def wake_instance(self):
        """Wake up a pending instance

        This method is called at the end of any session from check_gmp.
        Get the oldest pending entries and wake them up.
        """
        # How many processes are currently running?
        number_instances = self.has_entries(pending=False)

        # How many pending entries are waiting?
        number_pending_instances = self.has_entries(pending=True)

        if (
            number_instances < MAX_RUNNING_INSTANCES
            and number_pending_instances > 0
        ):
            pending_entries = self.get_oldest_pending_entries(
                MAX_RUNNING_INSTANCES - number_instances
            )

            logger.debug(
                "wake_instance: %i %i",
                number_instances,
                number_pending_instances,
            )

            for entry in pending_entries:
                created_at = entry[0]
                pid = entry[1]
                # Change status to not pending and continue the process
                self.update_pending_status(created_at, False)
                self.start_process(pid)

    def start_process(self, pid):
        """Continue a stopped process

        Send a continue signal to the process with given pid

        Arguments:
            pid (int): Process Identification Number
        """
        logger.debug("Continue pid: %i", pid)
        os.kill(pid, signal.SIGCONT)

    def stop_process(self, pid):
        """Stop a running process

        Send a stop signal to the process with given pid

        Arguments:
            pid (int): Process Identification Number
        """
        os.kill(pid, signal.SIGSTOP)

    def check_pid(self, pid):
        """Check for the existence of a process.

        Arguments:
            pid (int): Process Identification Number
        """
        try:
            os.kill(pid, 0)
        except OSError:
            return False
        else:
            return True


def ping(gmp, im):
    """Checks for connectivity

    This function sends the get_version command and checks whether the status
    is ok or not.
    """
    version = gmp.get_version()
    version_status = version.xpath("@status")

    if "200" in version_status:
        end_session(im, "GMP OK: Ping successful", NAGIOS_OK)
    else:
        end_session(im, "GMP CRITICAL: Machine dead?", NAGIOS_CRITICAL)


def status(gmp, im, script_args):
    """Returns the current status of a host

    This functions return the current state of a host.
    Either directly over the host management or within a task.

    For a task you can explicitly ask for the trend.
    Otherwise the last report of the task will be filtered.

    In the host management the report id in the details is taken
    as report for the filter.
    If the host information contains any vulnerabilities, then will the
    report be filtered too. With additional parameters it is possible to add
    more information about the vulnerabilities.

    * DFN-Certs
    * Logs
    * Autofp
    * Scanend
    * Overrides
    """
    params_used = (
        f"task={script_args.task} autofp={script_args.autofp} "
        f"overrides={script_args.overrides} "
        f"apply_overrides={script_args.apply_overrides}"
    )

    if script_args.task:
        task = gmp.get_tasks(
            filter_string=(
                "permission=any owner=any rows=1 " f'name="{script_args.task}"'
            )
        )
        if script_args.trend:
            trend = task.xpath("task/trend/text()")

            if not trend:
                end_session(
                    im, "GMP UNKNOWN: Trend is not available.", NAGIOS_UNKNOWN
                )

            trend = trend[0]

            if trend in ["up", "more"]:
                end_session(
                    im, f"GMP CRITICAL: Trend is {trend}.", NAGIOS_CRITICAL
                )
            elif trend in ["down", "same", "less"]:
                end_session(im, f"GMP OK: Trend is {trend}.", NAGIOS_OK)
            else:
                end_session(
                    im,
                    f"GMP UNKNOWN: Trend is unknown: {trend}",
                    NAGIOS_UNKNOWN,
                )
        else:
            last_report_id = task.xpath("task/last_report/report/@id")

            if not last_report_id:
                end_session(
                    im, "GMP UNKNOWN: Report is not available", NAGIOS_UNKNOWN
                )

            last_report_id = last_report_id[0]
            last_scan_end = task.xpath(
                "task/last_report/report/scan_end/text()"
            )

            if last_scan_end:
                last_scan_end = last_scan_end[0]
            else:
                last_scan_end = ""

            if im.is_old_report(last_scan_end, params_used):
                host = script_args.hostaddress

                full_report = gmp.get_report(
                    report_id=last_report_id,
                    filter_string=(
                        "sort-reverse=id result_hosts_only=1 min_cvss_base= "
                        f"min_qod= levels=hmlgd autofp={script_args.autofp} "
                        "notes=0 "
                        f"apply_overrides={script_args.apply_overrides} "
                        f"overrides={script_args.overrides} first=1 rows=-1 "
                        f"delta_states=cgns host={host}"
                    ),
                    details=True,
                )

                im.add_report(last_scan_end, params_used, full_report)
                logger.debug("Report added to db")
            else:
                full_report = im.load_local_report()

            filter_report(
                im, full_report.xpath("report/report")[0], script_args
            )


def filter_report(im, report, script_args):
    """Filter out the information in a report

    This function filters the results of a given report.

    Arguments:
        report (obj): Report as lxml ElementTree.
    """
    report_id = report.xpath("@id")
    if report_id:
        report_id = report_id[0]
    results = report.xpath("//results")
    if not results:
        end_session(
            im, "GMP UNKNOWN: Failed to get results list", NAGIOS_UNKNOWN
        )

    results = results[0]
    # Init variables
    any_found = False
    high_count = 0
    medium_count = 0
    low_count = 0
    log_count = 0
    error_count = 0

    nvts = {"high": [], "medium": [], "low": [], "log": []}

    all_results = results.xpath("result")

    for result in all_results:
        if script_args.hostaddress:
            host = result.xpath("host/text()")
            if not host:
                end_session(
                    im,
                    "GMP UNKNOWN: Failed to parse result host",
                    NAGIOS_UNKNOWN,
                )

            if script_args.hostaddress != host[0]:
                continue
            any_found = True

        threat = result.xpath("threat/text()")
        if not threat:
            end_session(
                im,
                "GMP UNKNOWN: Failed to parse result threat.",
                NAGIOS_UNKNOWN,
            )

        threat = threat[0]
        if threat in "High":
            high_count += 1
            if script_args.oid:
                nvts["high"].append(retrieve_nvt_data(result))
        elif threat in "Medium":
            medium_count += 1
            if script_args.oid:
                nvts["medium"].append(retrieve_nvt_data(result))
        elif threat in "Low":
            low_count += 1
            if script_args.oid:
                nvts["low"].append(retrieve_nvt_data(result))
        elif threat in "Log":
            log_count += 1
            if script_args.oid:
                nvts["log"].append(retrieve_nvt_data(result))
        else:
            end_session(
                im,
                f"GMP UNKNOWN: Unknown result threat: {threat}",
                NAGIOS_UNKNOWN,
            )

    errors = report.xpath("errors")

    if errors:
        errors = errors[0]
        if script_args.hostaddress:
            for error in errors.xpath("error"):
                host = error.xpath("host/text()")
                if script_args.hostaddress == host[0]:
                    error_count += 1
        else:
            error_count = errors.xpath("count/text()")[0]

    ret = 0
    if high_count > 0:
        ret = NAGIOS_CRITICAL
    elif medium_count > 0:
        ret = NAGIOS_WARNING

    if script_args.empty_as_unknown and (
        not all_results or (not any_found and script_args.hostaddress)
    ):
        ret = NAGIOS_UNKNOWN

    print(
        f"GMP {NAGIOS_MSG[ret]}: "
        f"{str((high_count + medium_count + low_count))} "
        f"vulnerabilities found - High: {str(high_count)} "
        f"Medium: {str(medium_count)} Low: {str(low_count)}"
    )

    if not all_results:
        print("Report did not contain any vulnerabilities")

    elif not any_found and script_args.hostaddress:
        print(
            "Report did not contain vulnerabilities "
            f"for IP {script_args.hostaddress}"
        )

    if int(error_count) > 0:
        if script_args.hostaddress:
            print_without_pipe(
                f"Report did contain {str(error_count)} "
                f"errors for IP {script_args.hostaddress}"
            )
        else:
            print_without_pipe(f"Report did contain {error_count} errors")

    if script_args.report_link:
        print(
            f"https://{script_args.hostname}/omp"
            f"?cmd=get_report&report_id={report_id}"
        )

    if script_args.oid:
        print_nvt_data(
            nvts,
            show_log=script_args.showlog,
            show_ports=script_args.show_ports,
            descr=script_args.descr,
            dfn=script_args.dfn,
        )

    if script_args.scanend:
        end = report.xpath("//end/text()")
        end = end[0] if end else "Timestamp of scan end not given"
        print(f"SCAN_END: {end}")

    if script_args.details:
        if script_args.hostname:
            print(f"GSM_Host: {script_args.hostname}:{str(script_args.port)}")
        if script_args.gmp_username:
            print(f"GMP_User: {script_args.gmp_username}")
        if script_args.task:
            print_without_pipe(f"Task: {script_args.task}")

    end_session(
        im,
        f"|High={str(high_count)} "
        f"Medium={str(medium_count)} "
        f"Low={str(low_count)}",
        ret,
    )


def retrieve_nvt_data(result):
    """Retrieve the nvt data out of the result object

    This function parse the xml tree to find the important nvt data.

    Arguments:
        result (obj): Result as lxml ElementTree

    Returns:
        Tuple -- List with oid, name, desc, port and dfn
    """
    oid = result.xpath("nvt/@oid")
    name = result.xpath("nvt/name/text()")
    desc = result.xpath("description/text()")
    port = result.xpath("port/text()")

    if oid:
        oid = oid[0]

    if name:
        name = name[0]

    if desc:
        desc = desc[0]
    else:
        desc = ""

    if port:
        port = port[0]
    else:
        port = ""

    certs = result.xpath("nvt/cert/cert_ref")

    dfn_list = []
    for ref in certs:
        ref_type = ref.xpath("@type")[0]
        ref_id = ref.xpath("@id")[0]

        if ref_type in "DFN-CERT":
            dfn_list.append(ref_id)

    return (oid, name, desc, port, dfn_list)


def print_nvt_data(
    nvts, show_log=False, show_ports=False, descr=False, dfn=False
):
    """Print nvt data

    Prints for each nvt found in the array the relevant data

    Arguments:
        nvts (obj): Object holding all nvts
    """
    for key, nvt_data in nvts.items():
        if key == "log" and not show_log:
            continue
        for nvt in nvt_data:
            print_without_pipe(f"NVT: {nvt[0]} ({key}) {nvt[1]}")
            if show_ports:
                print_without_pipe(f"PORT: {nvt[3]}")
            if descr:
                print_without_pipe(f"DESCR: {nvt[2]}")

            if dfn and nvt[4]:
                dfn_list = ", ".join(nvt[4])
                if dfn_list:
                    print_without_pipe(f"DFN-CERT: {dfn_list}")


def end_session(im, msg, nagios_status):
    """End the session

    Close the socket if open and print the last msg

    Arguments:
        msg string): Message to print
        nagios_status (int): Exit status
    """
    print(msg)

    # Delete this instance
    im.delete_instance()

    # Activate some waiting instances if possible
    im.wake_instance()

    # Close the connection to database
    im.close_db()

    sys.exit(nagios_status)


def print_without_pipe(msg):
    """Prints the message, but without any pipe symbol

    If any pipe symbol is in the msg string, then it will be replaced with
    broken pipe symbol.

    Arguments:
        msg (string): Message to print
    """
    if "|" in msg:
        msg = msg.replace("|", "¦")

    print(msg)


# ISO 8601 date time string parsing

# Copyright (c) 2007 - 2015 Michael Twomey

# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:

# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

__all__ = ["parse_date", "ParseError", "UTC"]

# Adapted from http://delete.me.uk/2005/03/iso8601.html
ISO8601_REGEX = re.compile(
    r"""
    (?P<year>[0-9]{4})
    (
        (
            (-(?P<monthdash>[0-9]{1,2}))
            |
            (?P<month>[0-9]{2})
            (?!$)  # Don't allow YYYYMM
        )
        (
            (
                (-(?P<daydash>[0-9]{1,2}))
                |
                (?P<day>[0-9]{2})
            )
            (
                (
                    (?P<separator>[ T])
                    (?P<hour>[0-9]{2})
                    (:{0,1}(?P<minute>[0-9]{2})){0,1}
                    (
                        :{0,1}(?P<second>[0-9]{1,2})
                        ([.,](?P<second_fraction>[0-9]+)){0,1}
                    ){0,1}
                    (?P<timezone>
                        Z
                        |
                        (
                            (?P<tz_sign>[-+])
                            (?P<tz_hour>[0-9]{2})
                            :{0,1}
                            (?P<tz_minute>[0-9]{2}){0,1}
                        )
                    ){0,1}
                ){0,1}
            )
        ){0,1}  # YYYY-MM
    ){0,1}  # YYYY only
    $
    """,
    re.VERBOSE,
)


class ParseError(Exception):
    """Raised when there is a problem parsing a date string"""


# Yoinked from python docs
ZERO = timedelta(0)


class Utc(tzinfo):
    """UTC Timezone"""

    def utcoffset(self, dt):
        return ZERO

    def tzname(self, dt):
        return "UTC"

    def dst(self, dt):
        return ZERO

    def __repr__(self):
        return "<iso8601.Utc>"


UTC = Utc()


class FixedOffset(tzinfo):
    """Fixed offset in hours and minutes from UTC"""

    def __init__(self, offset_hours, offset_minutes, name):
        self.__offset_hours = offset_hours  # Keep for later __getinitargs__
        # Keep for later __getinitargs__
        self.__offset_minutes = offset_minutes
        self.__offset = timedelta(hours=offset_hours, minutes=offset_minutes)
        self.__name = name

    def __eq__(self, other):
        if isinstance(other, FixedOffset):
            # pylint: disable=protected-access
            return (other.__offset == self.__offset) and (
                other.__name == self.__name
            )
        if isinstance(other, tzinfo):
            return other == self
        return False

    def __getinitargs__(self):
        return (self.__offset_hours, self.__offset_minutes, self.__name)

    def utcoffset(self, dt):
        return self.__offset

    def tzname(self, dt):
        return self.__name

    def dst(self, dt):
        return ZERO

    def __repr__(self):
        return f"<FixedOffset {self.__name} {self.__offset}>"


def to_int(
    source_dict, key, default_to_zero=False, default=None, required=True
):
    """Pull a value from the dict and convert to int

    :param default_to_zero: If the value is None or empty, treat it as zero
    :param default: If the value is missing in the dict use this default

    """

    value = source_dict.get(key)
    if value in [None, ""]:
        value = default
    if (value in ["", None]) and default_to_zero:
        return 0
    if value is None:
        if required:
            raise ParseError(f"Unable to read {key} from {source_dict}")
        return value
    else:
        return int(value)


def parse_timezone(matches, default_timezone=UTC):
    """Parses ISO 8601 time zone specs into tzinfo offsets"""

    if matches["timezone"] == "Z":
        return UTC
    # This isn't strictly correct, but it's common to encounter dates without
    # timezones so I'll assume the default (which defaults to UTC).
    # Addresses issue 4.
    if matches["timezone"] is None:
        return default_timezone
    sign = matches["tz_sign"]
    hours = to_int(matches, "tz_hour")
    minutes = to_int(matches, "tz_minute", default_to_zero=True)
    description = f"{sign}{str(hours)}:{str(minutes)}"
    if sign == "-":
        hours = -1 * hours
        minutes = -1 * minutes
    return FixedOffset(hours, minutes, description)


def parse_date(datestring, default_timezone=UTC):
    """Parses ISO 8601 dates into datetime objects

    The timezone is parsed from the date string. However it is quite common to
    have dates without a timezone (not strictly correct). In this case the
    default timezone specified in default_timezone is used. This is UTC by
    default.

    Arguments
        datestring: The date to parse as a string
        default_timezone: A datetime tzinfo instance to use when no timezone
                          is specified in the datestring. If this is set to
                          None then a naive datetime object is returned.
    Returns:
        A datetime.datetime instance
    Raises:
        ParseError when there is a problem parsing the date or
        constructing the datetime instance.

    """
    if not isinstance(datestring, str):
        raise ParseError(f"Expecting a string {datestring}")

    match = ISO8601_REGEX.match(datestring)
    if not match:
        raise ParseError(f"Unable to parse date string {datestring}")

    groups = match.groupdict()

    tz = parse_timezone(groups, default_timezone=default_timezone)

    groups["second_fraction"] = int(
        Decimal(f"0.{groups['second_fraction'] or 0}") * Decimal("1000000.0")
    )

    try:
        return datetime(
            year=to_int(groups, "year"),
            month=to_int(
                groups,
                "month",
                default=to_int(groups, "monthdash", required=False, default=1),
            ),
            day=to_int(
                groups,
                "day",
                default=to_int(groups, "daydash", required=False, default=1),
            ),
            hour=to_int(groups, "hour", default_to_zero=True),
            minute=to_int(groups, "minute", default_to_zero=True),
            second=to_int(groups, "second", default_to_zero=True),
            microsecond=groups["second_fraction"],
            tzinfo=tz,
        )
    except Exception as e:
        raise ParseError(e) from None


def main(gmp: Gmp, args: Namespace) -> None:
    tmp_path = f"{tempfile.gettempdir()}/check_gmp/"
    tmp_path_db = tmp_path + "reports.db"

    prog = "check-gmp"

    parser = ArgumentParser(
        prog=prog,
        prefix_chars="-",
        description=HELP_TEXT,
        formatter_class=RawTextHelpFormatter,
        add_help=False,
        epilog="""
        usage: gvm-script [connection_type] check-gmp.gmp.py ...
        or: gvm-script [connection_type] check-gmp.gmp.py -H
        or: gvm-script connection_type --help""",
    )

    parser.add_argument(
        "-H", action="help", help="Show this help message and exit."
    )

    parser.add_argument(
        "-V",
        "--version",
        action="version",
        version=f"{prog} {__version__}",
        help="Show program's version number and exit",
    )

    parser.add_argument(
        "--cache",
        nargs="?",
        default=tmp_path_db,
        help=f"Path to cache file. Default: {tmp_path_db}.",
    )

    parser.add_argument(
        "--clean", action="store_true", help="Activate to clean the database."
    )

    parser.add_argument(
        "-u", "--gmp-username", help="GMP username.", required=False
    )

    parser.add_argument(
        "-w", "--gmp-password", help="GMP password.", required=False
    )

    parser.add_argument(
        "-F",
        "--hostaddress",
        required=False,
        default="",
        help="Report last report status of host <ip>.",
    )

    parser.add_argument(
        "-T", "--task", required=False, help="Report status of task <task>."
    )

    parser.add_argument(
        "--apply-overrides", action="store_true", help="Apply overrides."
    )

    parser.add_argument(
        "--overrides", action="store_true", help="Include overrides."
    )

    parser.add_argument(
        "-d",
        "--details",
        action="store_true",
        help="Include connection details in output.",
    )

    parser.add_argument(
        "-l",
        "--report-link",
        action="store_true",
        help="Include URL of report in output.",
    )

    parser.add_argument(
        "--dfn",
        action="store_true",
        help="Include DFN-CERT IDs on vulnerabilities in output.",
    )

    parser.add_argument(
        "--oid",
        action="store_true",
        help="Include OIDs of NVTs finding vulnerabilities in output.",
    )

    parser.add_argument(
        "--descr",
        action="store_true",
        help="Include descriptions of NVTs finding vulnerabilities in output.",
    )

    parser.add_argument(
        "--showlog", action="store_true", help="Include log messages in output."
    )

    parser.add_argument(
        "--show-ports",
        action="store_true",
        help="Include port of given vulnerable nvt in output.",
    )

    parser.add_argument(
        "--scanend",
        action="store_true",
        help="Include timestamp of scan end in output.",
    )

    parser.add_argument(
        "--autofp",
        type=int,
        choices=[0, 1, 2],
        default=0,
        help="Trust vendor security updates for automatic false positive"
        " filtering (0=No, 1=full match, 2=partial).",
    )

    parser.add_argument(
        "-e",
        "--empty-as-unknown",
        action="store_true",
        help="Respond with UNKNOWN on empty results.",
    )

    parser.add_argument(
        "-I",
        "--max-running-instances",
        default=10,
        type=int,
        help="Set the maximum simultaneous processes of check-gmp",
    )

    parser.add_argument("--hostname", nargs="?", required=False)

    group = parser.add_mutually_exclusive_group(required=False)
    group.add_argument(
        "--ping", action="store_true", help="Ping the gsm appliance."
    )

    group.add_argument(
        "--status", action="store_true", help="Report status of task."
    )

    group = parser.add_mutually_exclusive_group(required=False)
    group.add_argument(
        "--days",
        type=int,
        help="Delete database entries that are older than" " given days.",
    )
    group.add_argument("--ip", help="Delete database entry for given ip.")

    group = parser.add_mutually_exclusive_group(required=False)
    group.add_argument(
        "--trend", action="store_true", help="Report status by trend."
    )
    group.add_argument(
        "--last-report",
        action="store_true",
        help="Report status by last report.",
    )

    script_args = parser.parse_args(args.script_args)

    aux_parser = ArgumentParser(
        prefix_chars="-", formatter_class=RawTextHelpFormatter
    )
    aux_parser.add_argument("--hostname", nargs="?", required=False)
    gvm_tool_args, _ = aux_parser.parse_known_args(sys.argv)
    if "hostname" in gvm_tool_args:
        script_args.hostname = gvm_tool_args.hostname

    # Set the max running instances variable
    if script_args.max_running_instances:
        # TODO should be passed as local variable instead of using a global one
        # pylint: disable=global-statement
        global MAX_RUNNING_INSTANCES
        MAX_RUNNING_INSTANCES = script_args.max_running_instances

    # Set the report manager
    if script_args.cache:
        tmp_path_db = script_args.cache
    im = InstanceManager(tmp_path_db, parser)

    # Check if command holds clean command
    if script_args.clean:
        if script_args.ip:
            logger.info("Delete entry with ip %s", script_args.ip)
            im.delete_entry_with_ip(script_args.ip)
        elif script_args.days:
            logger.info("Delete entries older than %s days", script_args.days)
            im.delete_older_entries(script_args.days)
        sys.exit(1)

    # Set the host
    im.set_host(script_args.hostaddress)

    # Check if no more than 10 instances of check-gmp runs simultaneously
    im.check_instances()

    try:
        gmp.get_version()
    except Exception as e:  # pylint: disable=broad-except
        end_session(im, f"GMP CRITICAL: {str(e)}", NAGIOS_CRITICAL)

    if script_args.ping:
        ping(gmp, im)

    if "status" in script_args:
        status(gmp, im, script_args)


if __name__ == "__gmp__":
    main(gmp, args)



# SPDX-FileCopyrightText: 2018 Henning Häcker
# SPDX-FileCopyrightText: 2019-2021 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

from argparse import ArgumentParser, Namespace, RawTextHelpFormatter
from typing import List

from gvm.protocols.gmp import Gmp

HELP_TEXT = """
        This script makes an E-Mail alert scan.

        Usage examples:
            $ gvm-script --gmp-username name --gmp-password pass ssh --hostname
            ... start-alert-scan.gmp.py +h
            ... start-alert-scan.gmp.py ++target-name ++hosts ++ports \
                    ++port-list-name +C +R +S
            ... start-alert-scan.gmp.py ++target-name ++hosts ++port-list-id \
                    +C ++recipient ++sender
            ... start-alert-scan.gmp.py ++target-id +C ++recipient ++sender
    """


def get_scan_config(gmp: Gmp, config: int, debug: bool = False):
    # get all configs of the openvas instance
    # filter for all rows!
    res = gmp.get_scan_configs(filter_string="rows=-1")

    if config < 0 or config > 4:
        raise ValueError("Wrong config identifier. Choose between [0,4].")
    # match the config abbreviation to accepted config names
    config_list = [
        "Full and fast",
        "Full and fast ultimate",
        "Full and very deep",
        "Full and very deep ultimate",
        "System Discovery",
    ]
    template_abbreviation_mapper = {
        0: config_list[0],
        1: config_list[1],
        2: config_list[2],
        3: config_list[3],
        4: config_list[4],
    }

    for conf in res.xpath("config"):
        cid = conf.xpath("@id")[0]
        name = conf.xpath("name/text()")[0]

        # get the config id of the desired template
        if template_abbreviation_mapper.get(config) == name:
            config_id = cid
            if debug:
                print(name + ": " + config_id)
            break

    return config_id


def get_target(
    gmp,
    target_name: str = None,
    hosts: List[str] = None,
    ports: str = None,
    port_list_name: str = None,
    port_list_id: str = None,
    debug: bool = False,
):
    if target_name is None:
        target_name = "target"
    targets = gmp.get_targets(filter_string=target_name)
    existing_targets = [""]
    for target in targets.findall("target"):
        existing_targets.append(str(target.find("name").text))
    counter = 0
    # iterate over existing targets and find a vacant targetName
    if target_name in existing_targets:
        while True:
            tmp_name = f"{target_name} ({str(counter)})"
            if tmp_name in existing_targets:
                counter += 1
            else:
                target_name = tmp_name
                break

    if debug:
        print(f"target name: {target_name}")

    if not port_list_id:
        existing_port_lists = [""]
        port_lists_tree = gmp.get_port_lists()
        for plist in port_lists_tree.findall("port_list"):
            existing_port_lists.append(str(plist.find("name").text))

        if port_list_name is None:
            port_list_name = "portlist"

        if port_list_name in existing_port_lists:
            counter = 0
            while True:
                tmp_name = f"{port_list_name} ({str(counter)})"
                if tmp_name in existing_port_lists:
                    counter += 1
                else:
                    port_list_name = tmp_name
                    break

        port_list = gmp.create_port_list(name=port_list_name, port_range=ports)
        # create port list
        port_list_id = port_list.xpath("@id")[0]
        if debug:
            print(f"New Portlist-name:\t{port_list_name}")
            print(f"New Portlist-id:  \t{str(port_list_id)}")

    # integrate port list id into create_target
    res = gmp.create_target(target_name, hosts=hosts, port_list_id=port_list_id)
    print(f"New target '{target_name}' created.")
    return res.xpath("@id")[0]


def get_alert(
    gmp: Gmp,
    sender_email: str,
    recipient_email: str,
    alert_name: str = None,
    debug: bool = False,
):
    # create alert if necessary
    alert_object = gmp.get_alerts(filter_string=f"name={alert_name}")
    alert = alert_object.xpath("alert")

    if len(alert) == 0:
        print(f"creating new alert {alert_name}")
        gmp.create_alert(
            name=alert_name,
            event=gmp.types.AlertEvent.TASK_RUN_STATUS_CHANGED,
            event_data={"status": "Done"},
            condition=gmp.types.AlertCondition.ALWAYS,
            method=gmp.types.AlertMethod.EMAIL,
            method_data={
                """Task '$n': $e

After the event $e,
the following condition was met: $c

This email escalation is configured to attach report format '$r'.
Full details and other report formats are available on the scan engine.

$t

Note:
This email was sent to you as a configured security scan escalation.
Please contact your local system administrator if you think you
should not have received it.
""": "message",
                "2": "notice",
                sender_email: "from_address",
                "[OpenVAS-Manager] Task": "subject",
                "c402cc3e-b531-11e1-9163-406186ea4fc5": "notice_attach_format",
                recipient_email: "to_address",
            },
        )

        alert_object = gmp.get_alerts(filter_string=f"name={recipient_email}")
        alert = alert_object.xpath("alert")

    alert_id = alert[0].get("id", "no id found")
    if debug:
        print(f"alert_id: {str(alert_id)}")

    return alert_id


def get_scanner(gmp: Gmp):
    res = gmp.get_scanners()
    scanner_ids = res.xpath("scanner/@id")
    return scanner_ids[1]  # "default scanner"


def create_and_start_task(
    gmp: Gmp,
    config_id: str,
    target_id: str,
    scanner_id: str,
    alert_id: str,
    alert_name: str,
    debug: bool = False,
) -> str:
    # Create the task
    task_name = f"Alert Scan for Alert {alert_name}"
    tasks = gmp.get_tasks(filter_string=f'name="{task_name}"')
    existing_tasks = tasks.findall("task")

    if existing_tasks:
        task_name = f"Alert Scan for Alert {alert_name} ({len(existing_tasks)})"
    task_comment = "Alert Scan"
    res = gmp.create_task(
        name=task_name,
        config_id=config_id,
        target_id=target_id,
        scanner_id=scanner_id,
        alert_ids=[alert_id],
        comment=task_comment,
    )

    # Start the task
    task_id = res.xpath("@id")[0]
    gmp.start_task(task_id)

    if debug:
        # Stop the task (for performance reasons)
        gmp.stop_task(task_id=task_id)
        print("Task stopped")

    return task_name


def parse_args(args: Namespace):  # pylint: disable=unused-argument
    parser = ArgumentParser(
        prefix_chars="+",
        add_help=False,
        formatter_class=RawTextHelpFormatter,
        description=HELP_TEXT,
    )

    parser.add_argument(
        "+h",
        "++help",
        action="help",
        help="Show this help message and exit.",
    )

    target = parser.add_mutually_exclusive_group(required=True)

    target.add_argument(
        "++target-id",
        type=str,
        dest="target_id",
        help="Use an existing target by target id",
    )

    target.add_argument(
        "++target-name",
        type=str,
        dest="target_name",
        help="Create a target by name",
    )

    parser.add_argument(
        "++hosts",
        nargs="+",
        dest="hosts",
        help="Host(s) for the new target",
    )

    ports = parser.add_mutually_exclusive_group()

    ports.add_argument(
        "++port-list-id",
        type=str,
        dest="port_list_id",
        help="An existing portlist id for the new target",
    )
    ports.add_argument(
        "++ports",
        type=str,
        dest="ports",
        help="Ports in the new target: e.g. T:80-80,8080",
    )

    parser.add_argument(
        "++port-list-name",
        type=str,
        dest="port_list_name",
        help="Name for the new portlist in the new target",
    )

    config = parser.add_mutually_exclusive_group()

    config.add_argument(
        "+C",
        "++scan-config",
        default=0,
        type=int,
        dest="config",
        help="Choose from existing scan config:"
        "\n  0: Full and fast"
        "\n  1: Full and fast ultimate"
        "\n  2: Full and very deep"
        "\n  3: Full and very deep ultimate"
        "\n  4: System Discovery",
    )

    config.add_argument(
        "++scan-config-id",
        type=str,
        dest="scan_config_id",
        help="Use existing scan config by id",
    )

    parser.add_argument(
        "++scanner-id",
        type=str,
        dest="scanner_id",
        help="Use existing scanner by id",
    )

    parser.add_argument(
        "+R",
        "++recipient",
        required=True,
        dest="recipient_email",
        type=str,
        help="Alert recipient E-Mail address",
    )

    parser.add_argument(
        "+S",
        "++sender",
        required=True,
        dest="sender_email",
        type=str,
        help="Alert senders E-Mail address",
    )

    parser.add_argument(
        "++alert-name",
        dest="alert_name",
        type=str,
        help="Optional Alert name",
    )

    script_args, _ = parser.parse_known_args()
    return script_args


def main(gmp: Gmp, args: Namespace) -> None:
    # pylint: disable=undefined-variable, unused-argument

    script_args = parse_args(args)

    # set alert_name to recipient email if no other name
    # is given
    if script_args.alert_name is None:
        script_args.alert_name = script_args.recipient_email

    # use existing config from argument
    if not script_args.scan_config_id:
        config_id = get_scan_config(gmp, script_args.config)
    else:
        config_id = script_args.scan_config_id

    # create new target or use existing one from id
    if not script_args.target_id:
        target_id = get_target(
            gmp,
            target_name=script_args.target_name,
            hosts=script_args.hosts,
            ports=script_args.ports,
            port_list_name=script_args.port_list_name,
            port_list_id=script_args.port_list_id,
        )
    else:
        target_id = script_args.target_id
    alert_id = get_alert(
        gmp,
        script_args.sender_email,
        script_args.recipient_email,
        script_args.alert_name,
    )
    if not script_args.scanner_id:
        scanner_id = get_scanner(gmp)
    else:
        scanner_id = script_args.scanner_id

    task_name = create_and_start_task(
        gmp, config_id, target_id, scanner_id, alert_id, script_args.alert_name
    )

    print(f"Task started: {task_name}\n")
    print("Script finished\n")


if __name__ == "__gmp__":
    main(gmp, args)



# SPDX-FileCopyrightText: 2021 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

import csv
import datetime
import json
import time
from argparse import ArgumentParser, Namespace, RawTextHelpFormatter
from pathlib import Path
from typing import Dict, Tuple

from cpe import CPE
from gvm.protocols.gmp import Gmp
from gvmtools.helper import error_and_exit, generate_uuid
from lxml import etree as e

HELP_TEXT = (
    "This script creates a cve report from a JSON document.\n"
    "The JSON document needs to be formatted like this: "
    "["
    "    {"
    '        "headings": ['
    '            "name",'
    '            "IP Address",'
    '            "IP range",'
    '            "Operating System",'
    '            "CPE String 23",'
    '            "Name",'
    '            "Full Version (version)",'
    '            "CPE String 23"'
    "        ],"
    "        ...,"
    '        "results": ['
    "            ["
    '                "foo",'
    '                "127.0.0.1",'
    '                "127.0.0.1/32",'
    '                "Some Windows",'
    '                "cpe:2.3:o:microsoft:some_windows:-:*:*:*:*:*:*:*",'
    "                ["
    '                    "Some Microsoftware",'
    "                    ."
    "                ],"
    "                ["
    '                    "0.1",'
    "                    ..."
    "                ],"
    "                ["
    '                    "cpe:2.3:a:microsoft:microsoftware:0.1:*:*:*:*:*:*:*",'
    "                    ..."
    "                ]"
    "            ],"
    "        ]"
    "    }"
    "]"
    " Usable with gvm-script (gvm-tools). Help: gvm-script -h"
)


class ProgressBar:
    def __init__(self, length: int, count: int, pl_name: str):
        self.length = length
        self.count = count
        self.current = 0
        self.start_time = datetime.datetime.now()
        self.entities = pl_name

        self.eta = "???"
        self.seq = ""
        self.end = ""

        self._print()
        self.seq = "\r"

    def _leading_zeros(self) -> str:
        return (len(str(self.count)) - len(str(self.current))) * " "

    def _bar(self):
        points = int(self.length * (self.current / self.count))
        return str("·" * points + " " * (self.length - points))

    def _print(self):
        print(
            f"{self.seq}[{self._bar()}] | "
            f"{self._leading_zeros()}{str(self.current)}/{str(self.count)} "
            f"{self.entities} processed. | "
            f"ETA: {self.eta}",
            flush=True,
            end=self.end,
        )

    def update(self, progressed):
        self.current = progressed
        elapsed = datetime.datetime.now() - self.start_time
        self.eta = str(elapsed / self.current * (self.count - self.current))
        self._print()

    def done(self):
        self.current = self.count
        self.eta = (
            f"Done! Time consumed: {datetime.datetime.now() - self.start_time}"
        )
        self.end = "\n"
        self._print()


class ListGenerator:
    """
    Creating the CPE to CVE list used for the report generation
    in this this script.
    """

    def __init__(self, gmp: Gmp, filename: Path, recreate: bool):
        self.gmp = gmp
        if filename.exists():
            if recreate:
                filename.unlink()
            else:
                error_and_exit(
                    f'The file "{filename}" already exists. '
                    "If you want to delete the old list and "
                    'recreate the list run with "++create-list '
                    f'recreate +f {filename}"'
                )
        self.file = open(filename, "w", encoding="utf-8")

    def _cpe_to_cve(self, resp):
        """Write the CPEs and CVEs to the list"""
        cve_tags = resp.findall("info")
        for cve_tag in cve_tags[
            :-1
        ]:  # -1 because the last info tag is a wrongy. :D
            cve = None
            cpes = None
            if "id" in cve_tag.attrib:
                cve = cve_tag.attrib["id"]
                cpes = cve_tag.find("cve").find("products").text
                cvss = cve_tag.find("cve").find("cvss").text
                if cpes:
                    for cpe in cpes.strip().split(" "):
                        print(
                            f"'{cpe}','{cve}','{cvss}'",
                            file=self.file,
                            end="\n",
                        )

    def create_cpe_list(self, step: int = 3000):
        """Creates a CPE to CVE list in a CSV format:
        'cpe', 'cve', 'cvss'
        The CPE's have a 1-to-1-relation to the CVE's
        so CPE's can appear more then once in this
        list

        step(int): How many CVEs will be requested from the GSM
                   in one request. Be careful with higher values.
                   You will need to set the default timeout in
                   gvm-tools higher if you set step >3000. A higher
                   step will make the list generation faster.
        """
        resp = self.gmp.get_info_list(
            info_type=self.gmp.types.InfoType.CVE, filter="rows=1"
        )
        count = resp.find("info_count").text

        first = 0
        count = int(count)
        print(f"Creating CPE to CVE list. Found {count} CVE's.")
        progress_bar = ProgressBar(length=100, count=count, pl_name="CVEs")
        while (first + step) < count:
            resp = self.gmp.get_info_list(
                info_type=self.gmp.types.InfoType.CVE,
                filter=f"rows={step} first={first}",
            )
            self._cpe_to_cve(resp)
            first = first + step
            progress_bar.update(progressed=first)

        # find the rest
        resp = self.gmp.get_info_list(
            info_type=self.gmp.types.InfoType.CVE,
            filter=f"rows={count - first} first={first}",
        )
        self._cpe_to_cve(resp)
        progress_bar.done()

        self.file.close()


class Report:
    def __init__(self, gmp):
        self.results = e.Element("results", {"start": "1", "max": "-1"})
        self.hosts = []
        self.report = None

        self.gmp = gmp

    def finish_report(self):
        report_format_id = "d5da9f67-8551-4e51-807b-b6a873d70e34"
        self.report_id = generate_uuid()
        self.report = e.Element(
            "report",
            {
                "id": self.report_id,
                "format_id": report_format_id,
                "extension": "xml",
                "content_type": "text/xml",
            },
        )
        owner_elem = e.SubElement(self.report, "owner")
        e.SubElement(owner_elem, "name").text = ""
        e.SubElement(self.report, "name").text = "Report created from JSON-File"

        inner_report = e.SubElement(
            self.report, "report", {"id": self.report_id}
        )
        ports_elem = e.SubElement(
            inner_report, "ports", {"start": "1", "max": "-1"}
        )

        inner_report.append(ports_elem)
        inner_report.append(self.results)
        inner_report.extend(self.hosts)
        self.report.append(inner_report)

    def send_report(self) -> str:
        the_time = time.strftime("%Y/%m/%d-%H:%M:%S")
        task_id = ""
        task_name = f"CVE_Scan_Report_{the_time}"

        res = self.gmp.create_container_task(
            name=task_name, comment="Created with gvm-tools."
        )

        task_id = res.xpath("//@id")[0]

        report = e.tostring(self.report)

        res = self.gmp.import_report(report, task_id=task_id, in_assets=True)

        return res.xpath("//@id")[0]

    def generate_host_detail(
        self,
        name,
        value,
        source_name=None,
        source_description=None,
        source_type=None,
    ):
        """Generating a host details xml element"""
        host_detail_elem = e.Element("detail")
        e.SubElement(host_detail_elem, "name").text = name
        e.SubElement(host_detail_elem, "value").text = value

        if source_name:
            source_elem = e.SubElement(host_detail_elem, "source")
            e.SubElement(source_elem, "name").text = source_name
            if source_type:
                e.SubElement(source_elem, "type").text = source_type
            if source_description:
                e.SubElement(source_elem, "description").text = (
                    source_description
                )

        return host_detail_elem

    def add_results(self, ip, hostname, cpes: Dict, cpeo, os, date_time):
        host_id = generate_uuid()
        source_name = "gvm-tools"
        date_format = "%Y-%m-%dT%H:%M:%S"
        date_time = f"{date_time.strftime(date_format)}Z"

        host_elem = e.Element("host")
        e.SubElement(host_elem, "ip").text = ip
        e.SubElement(host_elem, "asset", {"asset_id": host_id})
        e.SubElement(host_elem, "start").text = date_time
        e.SubElement(host_elem, "end").text = date_time
        host_result_count_elem = e.SubElement(host_elem, "result_count")
        host_elem.append(
            self.generate_host_detail(
                name="hostname", value=hostname, source_name=source_name
            )
        )
        host_elem.append(
            self.generate_host_detail(
                name="best_os_txt",
                value=os,
                source_name=source_name,
                source_description="Host Details",
            )
        )
        host_elem.append(
            self.generate_host_detail(
                name="best_os_cpe",
                value=cpeo,
                source_name=source_name,
                source_description="Host Details",
            )
        )

        host_details = 0
        for cpe, cves in cpes.items():
            if cves:
                for cve, cvss in cves.items():
                    result_id = generate_uuid()
                    result = e.Element("result", {"id": result_id})
                    e.SubElement(result, "name").text = f"Result for host {ip}"
                    e.SubElement(result, "comment").text = (
                        "Imported with gvm-tools"
                    )
                    e.SubElement(result, "modification_time").text = date_time
                    e.SubElement(result, "creation_time").text = date_time
                    detect_elem = e.Element("detection")
                    detect_result_elem = e.SubElement(
                        detect_elem, "result", {"id": result_id}
                    )
                    details_elem = e.SubElement(detect_result_elem, "details")
                    # We need to add the detection details here
                    # but actually they are not imported to GSM anyways ...
                    e.SubElement(details_elem, "detail")

                    result_host_elem = e.Element("host")
                    result_host_elem.text = ip
                    e.SubElement(
                        result_host_elem, "asset", {"asset_id": host_id}
                    )
                    e.SubElement(result_host_elem, "hostname").text = hostname
                    result.append(result_host_elem)

                    nvt_elem = e.Element("nvt", {"oid": cve})
                    e.SubElement(nvt_elem, "type").text = "cve"
                    e.SubElement(nvt_elem, "name").text = cve
                    e.SubElement(nvt_elem, "cvss_base").text = str(cvss)
                    e.SubElement(nvt_elem, "cve").text = cve

                    result.append(nvt_elem)

                    e.SubElement(result, "severity").text = str(cvss)

                    host_elem.append(
                        self.generate_host_detail(
                            name="App",
                            value=cpe,
                            source_type="cve",
                            source_name=cve,
                            source_description="CVE Scanner",
                        )
                    )
                    host_details = host_details + 1
                    self.results.append(result)
        e.SubElement(host_result_count_elem, "page").text = str(host_details)
        self.hosts.append(host_elem)


class Parser:
    """Class handles the Parsing from JSON to a Report"""

    def __init__(self, gmp: Gmp, json_file: Path, cpe_list: Path) -> None:
        try:
            self.cpe_list = open(cpe_list, "r", encoding="utf-8")
            self.reader = csv.reader(self.cpe_list)
        except FileNotFoundError:
            error_and_exit(
                f'There is no file "{cpe_list}". '
                "Maybe you need to create a list first. Run with "
                f'argument "++create-list +f {cpe_list}", to create '
                "a new list, or pass the correct location of an existing list."
            )
        self.gmp = gmp
        try:
            self.json_fp = open(json_file, encoding="utf-8")
            self.json_dump = json.load(self.json_fp)[0]["results"]
        except FileNotFoundError:
            error_and_exit(f'There is no file "{json_file}".')
        except json.JSONDecodeError as exc:
            error_and_exit(f"The JSON seems to be invalid: {exc.args[0]}")

    def parse(self) -> Report:
        """Loads an JSON file and extracts host information:

        Args:
            host_dump: the dumped json results, containing a hostname,
                    host_ip, host_ip_range, host_operating_system,
                    host_os_cpe, arrays of found_app, app_version,
                    app_cpe
        """

        report = Report(gmp=gmp)

        date_time = datetime.datetime.now()

        count = len(self.json_dump)
        progressed = 0
        print(f"Found {str(count)} hosts:")

        progressbar = ProgressBar(length=100, count=count, pl_name="Hosts")

        for entry in self.json_dump:
            if entry[3] is None:
                error_and_exit("The JSON format is not correct.")
            name = entry[0]
            # print(f"Creating Results for the host {name}")
            ips = entry[1]
            if isinstance(ips, str):
                ips = [ips]
            os = entry[3]
            os_cpe = convert_cpe23_to_cpe22(entry[4])[0]

            cpes = []
            # entry[7] should be the CPEs ...
            if entry[7] is not None:
                if isinstance(entry[7], str):
                    cpes.extend(self._get_cpes(entry[7]))
                else:
                    for cpe in entry[7]:
                        if cpe:
                            cpes.extend(self._get_cpes(cpe))

            vulns = self._get_cves(cpes)
            if vulns:
                for ip in ips:
                    report.add_results(
                        ip=ip,
                        hostname=name,
                        cpes=vulns,
                        cpeo=os_cpe,
                        os=os,
                        date_time=date_time,
                    )

            progressed += 1
            progressbar.update(progressed=progressed)

        progressbar.done()
        return report

    def _get_cpes(self, cpe):
        """Parse and return the CPE's from the JSON.
        Convert the CPEs to v2.2 and check if they have a
        version part. If not get this CPE in all versions
        from the GSM and return them. This may result in
        a lot of false positives or false negatives.
        """
        cpe = convert_cpe23_to_cpe22(cpe)
        if cpe[1] is False:
            return [cpe[0]]

        cpes = []
        cpe_xml = self.gmp.get_info_list(
            info_type=gmp.types.InfoType.CPE, filter=f'rows=-1 uuid~"{cpe[0]}:"'
        )
        infos = cpe_xml.findall("info")
        for cpe in infos[:-1]:  # -1 because the last info tag is a wrongy. :D
            cpes.append(cpe.get("id"))
        return cpes

    def _get_cves(self, cpes):
        """Get CVEs for the CPEs from the CSV List"""
        vulns = {}
        i = 0
        for cpe in cpes:
            vulns[cpe] = {}
        for row in self.reader:  # O(n)
            for cpe in cpes:
                if cpe in row[0]:
                    vulns[cpe][row[1].strip("'")] = float(row[2].strip("'"))
                    i = i + 1
        self.cpe_list.seek(0)

        return vulns

    def finish_lookup(self):
        self.json_fp.close()
        self.cpe_list.close()


def convert_cpe23_to_cpe22(cpe: str) -> Tuple[str, bool]:
    """Convert a CPE v2.3 to a CPE v2.2
    returns the CPE v2.2 and True if no product
    version is given
    """
    # MAKE ME BETTER!!!
    cpe = CPE(cpe)
    any_version = False
    if cpe.get_version()[0] == "*":
        any_version = True
    return (
        str(CPE(cpe.as_uri_2_3(), CPE.VERSION_2_2)).replace("CPE v2.2: ", ""),
        any_version,
    )


def parse_args(args: Namespace) -> Namespace:  # pylint: disable=unused-argument
    """Parsing args ..."""

    parser = ArgumentParser(
        prefix_chars="+",
        add_help=False,
        formatter_class=RawTextHelpFormatter,
        description=HELP_TEXT,
    )

    parser.add_argument(
        "+h",
        "++help",
        action="help",
        help="Show this help message and exit.",
    )

    parser.add_argument(
        "++create-list",
        nargs="?",
        type=str,
        choices=("no_creation", "recreate", "create"),
        const="create",
        default="no_creation",
        dest="create_list",
        help="Create the CPE to CVE helper list",
    )

    parser.add_argument(
        "+l",
        "++list",
        type=str,
        dest="list",
        required=True,
        help="Create the CPE to CVE helper list",
    )

    parser.add_argument(
        "+f",
        "++file",
        type=str,
        dest="json_file",
        help="File that should be parsed",
    )

    args, _ = parser.parse_known_args()

    return args


def main(gmp: Gmp, args: Namespace) -> None:
    # pylint: disable=undefined-variable

    parsed_args = parse_args(args=args)

    recreate = False
    if parsed_args.create_list == "recreate":
        recreate = True
    if parsed_args.create_list != "no_creation":
        print("Generating CPE to CVE list.")
        list_generator = ListGenerator(
            gmp, filename=Path(parsed_args.list).absolute(), recreate=recreate
        )
        list_generator.create_cpe_list()
        print("Generation of CPE to CVE list done.")
    if parsed_args.json_file:
        report = Parser(
            gmp=gmp, json_file=parsed_args.json_file, cpe_list=parsed_args.list
        ).parse()

        report.finish_report()
        report_id = report.send_report()
        print(f"Imported Report [{report_id}]")


if __name__ == "__gmp__":
    main(gmp, args)



# -*- coding: utf-8 -*-
# SPDX-FileCopyrightText: 2019-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
Module to store gvm-tools configuration settings
"""

import configparser
import logging

from gvm.connections import (
    DEFAULT_GVM_PORT,
    DEFAULT_HOSTNAME,
    DEFAULT_SSH_PORT,
    DEFAULT_UNIX_SOCKET_PATH,
)

logger = logging.getLogger(__name__)


class Config:
    def __init__(self):
        self._config = configparser.ConfigParser(default_section="main")

        self._config = {}

        self._config["gmp"] = dict(username="", password="")
        self._config["ssh"] = dict(
            username="gmp",
            password="gmp",
            port=DEFAULT_SSH_PORT,
            hostname=DEFAULT_HOSTNAME,
        )
        self._config["unixsocket"] = dict(socketpath=DEFAULT_UNIX_SOCKET_PATH)
        self._config["tls"] = dict(
            port=DEFAULT_GVM_PORT, hostname=DEFAULT_HOSTNAME
        )

        self._defaults = dict()

    def load(self, filepath):
        path = filepath.expanduser()

        config = configparser.ConfigParser(default_section="main")

        with path.open() as f:
            config.read_file(f)

        if "Auth" in config:
            logger.warning(
                "Warning: Loaded config file %s contains deprecated 'Auth' "
                "section. This section will be ignored in future.",
                str(filepath),
            )
            gmp_username = config.get("Auth", "gmp_username", fallback="")
            gmp_password = config.get("Auth", "gmp_password", fallback="")
            self._config["gmp"]["username"] = gmp_username
            self._config["gmp"]["password"] = gmp_password

        self._defaults.update(config.defaults())

        for section in config.sections():
            if section == "Auth":
                continue

            for key, value in config.items(section):
                self._config.setdefault(section, dict())[key] = value

    def defaults(self):
        return self._defaults

    def get(self, section, name):
        if section not in self._config:
            return None

        return self._config[section].get(name)



# -*- coding: utf-8 -*-
# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import sys
import traceback
from argparse import Namespace

from gvm import get_version as get_gvm_version
from gvm.protocols.gmp import Gmp
from gvm.protocols.latest import Osp
from gvm.transforms import EtreeCheckCommandTransform

from gvmtools import get_version
from gvmtools.helper import authenticate, do_not_run_as_root, run_script
from gvmtools.parser import (
    PROTOCOL_GMP,
    PROTOCOL_OSP,
    create_connection,
    create_parser,
)

HELP_TEXT = """
    Command line tool to execute custom GMP (Greenbone Management
    Protocol) and OSP (Open Scanner Protocol) scripts.

    The protocol specifications for GMP and OSP are available at:
      https://docs.greenbone.net/index.html#api_documentation
"""

__version__ = get_version()
__api_version__ = get_gvm_version()


def main():
    do_not_run_as_root()

    parser = create_parser(description=HELP_TEXT, logfilename="gvm-script.log")

    parser.add_protocol_argument()

    parser.add_argument(
        "scriptname",
        metavar="SCRIPT",
        help="Path to script to be executed (example: myscript.gmp.py)",
    )
    parser.add_argument(
        "scriptargs", nargs="*", metavar="ARG", help="Arguments for the script"
    )
    args, script_args = parser.parse_known_args()

    connection = create_connection(**vars(args))

    transform = EtreeCheckCommandTransform()

    global_vars = {
        "__version__": __version__,
        "__api_version__": __api_version__,
    }

    username = None
    password = None

    if args.protocol == PROTOCOL_OSP:
        protocol_class = Osp
        name = "osp"
    else:
        protocol_class = Gmp
        name = "gmp"

    try:
        with protocol_class(connection, transform=transform) as protocol:
            global_vars[name] = protocol
            global_vars["__name__"] = f"__{name}__"

            if args.protocol == PROTOCOL_GMP:
                if args.gmp_username:
                    (username, password) = authenticate(
                        protocol,
                        username=args.gmp_username,
                        password=args.gmp_password,
                    )

            argv = [os.path.abspath(args.scriptname), *args.scriptargs]

            shell_args = Namespace(
                username=username,
                password=password,
                argv=argv,
                # for backwards compatibility we add script here
                script=argv,
                # the unknown args, which are owned by the script.
                script_args=script_args,
            )

            global_vars["args"] = shell_args

            run_script(args.scriptname, global_vars)

    except Exception:  # pylint: disable=broad-except
        print(traceback.format_exc())
        sys.exit(1)

    sys.exit(0)


if __name__ == "__main__":
    main()



# SPDX-FileCopyrightText: 2017-2021 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

import sys
from argparse import Namespace

from gvm.protocols.gmp import Gmp


def check_args(args):
    len_args = len(args.script) - 1
    if len_args != 1:
        message = """
        This script will display all hosts with the searched applications!

        1. <application>  -- Name of the application

        Example:
            $ gvm-script --gmp-username name --gmp-password pass \
    ssh --hostname <gsm> scripts/application-detection.gmp.py <application>
        """
        print(message)
        sys.exit()


def print_assets(gmp, appname):
    res = gmp.get_reports(details=False)

    reports = res.xpath("/get_reports_response/report")
    for report in reports:
        report_id = report.attrib["id"]
        print_assets_for_host(gmp, appname, report_id)


def print_assets_for_host(gmp, appname, report_id):
    res = gmp.get_report(
        report_id, details=True, filter_string="rows=1 result_hosts_only=0"
    )

    hosts = res.xpath("/get_reports_response/report/report/host")

    for host in hosts:
        ip = host.xpath("ip/text()")
        if len(ip) == 0:
            continue
        else:
            ip = ip[0]

        hostname = host.xpath('detail/name[text()="hostname"]/../value/text()')
        if len(hostname) == 0:
            hostname = ""
        else:
            hostname = hostname[0]

        apps = host.xpath(
            'detail/name[text() = "App"]/../value['
            f'contains(text(), "{appname}")]/text()'
        )
        if len(apps) == 0:
            continue

        print(f"{ip} ({hostname})")
        for app in apps:
            print("\t" + app)
        print("\n")


def main(gmp: Gmp, args: Namespace) -> None:
    # pylint: disable=undefined-variable

    check_args(args)

    print_assets(gmp, args.script[1])


if __name__ == "__gmp__":
    main(gmp, args)



# SPDX-FileCopyrightText: 2014 Martin Boller
#
# SPDX-License-Identifier: GPL-3.0-or-later

from argparse import Namespace

from gvm.protocols.gmp import Gmp


def main(gmp: Gmp, args: Namespace) -> None:

    print("Emptying Trash...\n")

    try:
        status_text = gmp.empty_trashcan().xpath("@status_text")[0]
        print(status_text)
    except Exception as e:
        print(f"{e=}")


if __name__ == "__gmp__":
    main(gmp, args)



# SPDX-FileCopyrightText: 2017-2021 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

import sys
from argparse import Namespace
from datetime import datetime

from gvm.errors import GvmError
from gvm.protocols.gmp import Gmp


def check_args(args):
    len_args = len(args.script) - 1
    if len_args != 2:
        message = """
        This script creates a new task with specific host and nvt!
        It needs two parameters after the script name.

        <oid>       -- oid of the nvt
        <target>    -- scan target

        Example:
            $ gvm-script --gmp-username name --gmp-password pass \
    ssh --hostname <gsm> 1.3.6.1.4.1.25623.1.0.106223 localhost
        """
        print(message)
        sys.exit()


def create_scan_config(gmp, nvt_oid) -> str:
    # Create new config
    copy_id = "085569ce-73ed-11df-83c3-002264764cea"
    config_name = nvt_oid
    config_id = ""

    try:
        res = gmp.create_scan_config(copy_id, config_name)
        config_id = res.xpath("@id")[0]

        # Modify the config with an nvt oid
        nvt = gmp.get_scan_config_nvt(nvt_oid)
        family = nvt.xpath("nvt/family/text()")[0]

        gmp.modify_scan_config_set_nvt_selection(
            config_id=config_id, nvt_oids=[nvt_oid], family=family
        )

        # This nvts must be present to work
        family = "Port scanners"
        nvts = ["1.3.6.1.4.1.25623.1.0.14259", "1.3.6.1.4.1.25623.1.0.100315"]
        gmp.modify_scan_config_set_nvt_selection(
            config_id, nvt_oids=nvts, family=family
        )

    except GvmError:
        res = gmp.get_scan_configs(filter_string=f"name={config_name}")
        config_id = res.xpath("config/@id")[0]

    return config_id


def create_target(gmp: Gmp, name: str) -> str:
    try:
        res = gmp.create_target(name, hosts=[name])
        target_id = res.xpath("@id")[0]
    except GvmError:
        res = gmp.get_targets(filter_string=f"name={name} hosts={name}")
        target_id = res.xpath("target/@id")[0]

    return target_id


def create_and_start_task(
    gmp: Gmp, name: str, nvt_oid: str, config_id: str, target_id: str
) -> None:
    # Standard Scanner OpenVAS Default
    scanner_id = "08b69003-5fc2-4037-a479-93b440211c73"
    date_time = datetime.now().strftime("%Y-%m-%d %H:%M:%S")
    # Create task
    task_name = f"{name}_{nvt_oid}_{date_time}"
    res = gmp.create_task(
        name=task_name,
        config_id=config_id,
        target_id=target_id,
        scanner_id=scanner_id,
    )
    task_id = res.xpath("@id")[0]

    # Start the task
    gmp.start_task(task_id=task_id)
    print(f"\nTask {task_id} started")


def main(gmp: Gmp, args: Namespace) -> None:
    # pylint: disable=undefined-variable

    check_args(args)

    nvt_oid = args.script[1]
    target_name = args.script[2]

    config_id = create_scan_config(gmp, nvt_oid)
    target_id = create_target(gmp, target_name)

    create_and_start_task(gmp, target_name, nvt_oid, config_id, target_id)


if __name__ == "__gmp__":
    main(gmp, args)



# -*- coding: utf-8 -*-
# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import logging
import sys
import time

from gvm.errors import GvmError
from gvm.protocols.gmp import Gmp
from gvm.protocols.latest import Osp
from gvm.transforms import CheckCommandTransform
from gvm.xml import parse_xml, pretty_print

from gvmtools.helper import authenticate, do_not_run_as_root
from gvmtools.parser import (
    PROTOCOL_GMP,
    PROTOCOL_OSP,
    create_connection,
    create_parser,
)

logger = logging.getLogger(__name__)

HELP_TEXT = """
    Command line tool to access services via GMP (Greenbone Management Protocol) and OSP (Open Scanner Protocol)

    Examples:
      gvm-cli socket --help
      gvm-cli tls --help
      gvm-cli ssh --help

      gvm-cli socket --xml "<get_version/>"
      gvm-cli socket --xml "<commands><authenticate><credentials><username>myuser</username><password>mypass</password></credentials></authenticate><get_tasks/></commands>"
      gvm-cli socket --gmp-username foo --gmp-password foo myfile.xml

    The protocol specifications for GMP and OSP are available at:
      https://docs.greenbone.net/index.html#api_documentation"""  # noqa: E501


def _load_infile(filename=None):
    if not filename:
        return None

    with open(filename, encoding="utf-8") as f:
        return f.read()


def main():
    do_not_run_as_root()

    parser = create_parser(description=HELP_TEXT, logfilename="gvm-cli.log")

    parser.add_protocol_argument()

    parser.add_argument("-X", "--xml", help="XML request to send")
    parser.add_argument(
        "-r", "--raw", help="Return raw XML", action="store_true", default=False
    )
    parser.add_argument(
        "--pretty",
        help="Pretty format the returned xml",
        action="store_true",
        default=False,
    )
    parser.add_argument(
        "--duration", action="store_true", help="Measure command execution time"
    )
    parser.add_argument(
        "infile", nargs="?", help="File to read XML commands from."
    )

    args = parser.parse_args()

    # If timeout value is -1, then the socket has no timeout for this session
    if args.timeout == -1:
        args.timeout = None

    if args.xml is not None:
        xml = args.xml
    else:
        try:
            xml = _load_infile(args.infile)
        except IOError as e:
            print(e, file=sys.stderr)
            sys.exit(1)

    # If no command was given, program asks for one
    if len(xml) == 0:
        xml = input()

    try:
        parse_xml(xml)
    except GvmError as e:
        print(e, file=sys.stderr)
        sys.exit(1)

    connection = create_connection(**vars(args))

    if args.raw:
        transform = None
    else:
        transform = CheckCommandTransform()

    if args.protocol == PROTOCOL_OSP:
        protocol_class = Osp
    else:
        protocol_class = Gmp

    try:
        with protocol_class(connection, transform=transform) as protocol:
            if args.protocol == PROTOCOL_GMP:
                # Ask for password if none are given
                authenticate(protocol, args.gmp_username, args.gmp_password)

            if args.duration:
                starttime = time.time()

            result = protocol.send_command(xml)

            if args.duration:
                duration = time.time() - starttime
                print(f"Elapsed time: {duration} seconds")
            elif args.pretty:
                pretty_print(result)
            else:
                print(result)

    except Exception as e:  # pylint: disable=broad-except
        logger.error(e)
        sys.exit(1)
    sys.exit(0)


if __name__ == "__main__":
    main()



# SPDX-FileCopyrightText: 2024 Martin Boller
#
# SPDX-License-Identifier: GPL-3.0-or-later

import sys
from argparse import ArgumentParser, Namespace, RawTextHelpFormatter

from gvm.protocols.gmp import Gmp
from gvmtools.helper import Table

HELP_TEXT = (
    "This script list reports with the status "
    "defined on the commandline. Status can be: \n"
    "Requested, Queued, Interrupted, Running, or Done \n"
    "Note: Case matters. E.g.  - done - won't work, but - Done - will"
)


def check_args(args):
    len_args = len(args.script) - 1
    if len_args != 1:
        message = """
        This script lists all reports depending on status.
        One parameter after the script name is required.

        1. Status -- Either \"All\", \"Requested\", \"Queued\", \"Interrupted\", \"Running\", \"StopRequested\", \"Stopped\", or \"Done\"

        Example:
            $ gvm-script --gmp-username name --gmp-password pass \
socket list-reports.gmp.py Done \n
        """
        print(message)
        sys.exit()


def parse_args(args: Namespace) -> Namespace:  # pylint: disable=unused-argument
    """Parsing args ..."""

    parser = ArgumentParser(
        prefix_chars="+",
        add_help=False,
        formatter_class=RawTextHelpFormatter,
        description=HELP_TEXT,
    )

    parser.add_argument(
        "+h",
        "++help",
        action="help",
        help="Show this help message and exit.",
    )

    parser.add_argument(
        "status_cmd",
        type=str,
        help=(
            'Status: "All", "Queued", "Requested", "Interrupted", "Running", "Stop Requested", "Stopped" or "Done"'
        ),
    )
    script_args, _ = parser.parse_known_args(args)
    return script_args


def list_reports(
    gmp: Gmp,
    status: str,
):
    str_status = status
    if status.upper() == "ALL":
        status = "All"
    elif status.upper() == "REQUESTED":
        str_status = "Requested"
    elif status.upper() == "INTERRUPTED":
        str_status = "Interrupted"
    elif status.upper() == "QUEUED":
        str_status = "Queued"
    elif status.upper()[:6] == "STOPRE":
        str_status = "Stop Requested"
    elif status.upper()[:6] == "STOP R":
        str_status = "Stop Requested"
    elif status.upper() == "DONE":
        str_status = "Done"
    elif status.upper() == "RUNNING":
        str_status = "Running"
    elif status.upper() == "STOPPED":
        str_status = "Stopped"
    else:
        str_status = "All"

    print("Reports with status: " + str_status + "\n")

    if str_status == "All":
        response_xml = gmp.get_reports(
            ignore_pagination=True, details=True, filter_string="rows=-1"
        )
    else:
        response_xml = gmp.get_reports(
            ignore_pagination=True,
            details=True,
            filter_string="status="
            + str_status
            + "  and sort-reverse=name and rows=-1",
        )

    reports_xml = response_xml.xpath("report")
    heading = [
        "#",
        "Id",
        "Creation Time",
        "Modification Time",
        "Task Name",
        "Status",
        "Progress",
    ]
    rows = []
    numberRows = 0

    for report in reports_xml:
        # Count number of reports
        numberRows = numberRows + 1
        # Cast/convert to text to show in list
        rowNumber = str(numberRows)
        creation_time = "".join(report.xpath("creation_time/text()"))
        # report_name = "".join(report.xpath("name/text()")) # Report name is the same as Creation Time
        report_id = report.get("id")
        report_task = "".join(report.xpath("task/name/text()"))
        mod_time = "".join(report.xpath("modification_time/text()"))
        report_status = "".join(report.xpath("report/scan_run_status/text()"))
        report_progress = (
            "".join(report.xpath("report/task/progress/text()")) + "%"
        )
        rows.append(
            [
                rowNumber,
                report_id,
                creation_time,
                mod_time,
                report_task,
                report_status,
                report_progress,
            ]
        )

    print(Table(heading=heading, rows=rows))


def main(gmp: Gmp, args: Namespace) -> None:
    # pylint: disable=unused-argument
    if args.script:
        args = args.script[1:]

    parsed_args = parse_args(args=args)

    print("Listing reports.\n")

    list_reports(gmp, parsed_args.status_cmd)


if __name__ == "__gmp__":
    main(gmp, args)



# -*- coding: utf-8 -*-
# SPDX-FileCopyrightText: 2019-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import sys
import unittest
from argparse import Namespace
from pathlib import Path
from unittest.mock import patch

from gvm.connections import (
    DEFAULT_TIMEOUT,
    DEFAULT_UNIX_SOCKET_PATH,
    SSHConnection,
    TLSConnection,
    UnixSocketConnection,
)

from gvmtools.parser import CliParser, create_connection, create_parser

from . import SuppressOutput

__here__ = Path(__file__).parent.resolve()


class ConfigParserTestCase(unittest.TestCase):
    def setUp(self):
        self.test_config_path = __here__ / "test.cfg"

        self.assertTrue(self.test_config_path.is_file())

        self.parser = CliParser("TestParser", "test.log")

    def test_socket_defaults_from_config(self):
        args = self.parser.parse_args(
            ["--config", str(self.test_config_path), "socket"]
        )

        self.assertEqual(args.foo, "bar")
        self.assertEqual(args.timeout, 1000)
        self.assertEqual(args.gmp_password, "bar")
        self.assertEqual(args.gmp_username, "bar")
        self.assertEqual(args.socketpath, "/foo/bar.sock")

    def test_ssh_defaults_from_config(self):
        args = self.parser.parse_args(
            ["--config", str(self.test_config_path), "ssh", "--hostname", "foo"]
        )

        self.assertEqual(args.foo, "bar")
        self.assertEqual(args.timeout, 1000)
        self.assertEqual(args.gmp_password, "bar")
        self.assertEqual(args.gmp_username, "bar")
        self.assertEqual(args.ssh_password, "lorem")
        self.assertEqual(args.ssh_username, "ipsum")
        self.assertEqual(args.port, 123)

    def test_tls_defaults_from_config(self):
        args = self.parser.parse_args(
            ["--config", str(self.test_config_path), "tls", "--hostname", "foo"]
        )

        self.assertEqual(args.foo, "bar")
        self.assertEqual(args.timeout, 1000)
        self.assertEqual(args.gmp_password, "bar")
        self.assertEqual(args.gmp_username, "bar")
        self.assertEqual(args.certfile, "foo.cert")
        self.assertEqual(args.keyfile, "foo.key")
        self.assertEqual(args.cafile, "foo.ca")
        self.assertEqual(args.port, 123)

    @patch("gvmtools.parser.logger")
    @patch("gvmtools.parser.Path")
    def test_resolve_file_not_found_error(self, path_mock, logger_mock):
        # Making sure that resolve raises an error
        def resolve_raises_error():
            raise FileNotFoundError()

        configpath = unittest.mock.MagicMock()
        configpath.expanduser().resolve = unittest.mock.MagicMock(
            side_effect=resolve_raises_error
        )
        path_mock.return_value = configpath

        logger_mock.debug = unittest.mock.MagicMock()

        args = self.parser.parse_args(["socket"])

        self.assertIsInstance(args, Namespace)
        self.assertEqual(args.connection_type, "socket")
        self.assertEqual(args.config, "~/.config/gvm-tools.conf")
        logger_mock.debug.assert_any_call(
            "Ignoring non existing config file %s", "~/.config/gvm-tools.conf"
        )

    @patch("gvmtools.parser.Path")
    @patch("gvmtools.parser.Config")
    def test_config_load_raises_error(self, config_mock, path_mock):
        def config_load_error():
            raise Exception  # pylint: disable=broad-exception-raised

        config = unittest.mock.MagicMock()
        config.load = unittest.mock.MagicMock(side_effect=config_load_error)
        config_mock.return_value = config

        # Making sure that the function thinks the config file exists
        configpath_exists = unittest.mock.Mock()
        configpath_exists.expanduser().resolve().exists = (
            unittest.mock.MagicMock(return_value=True)
        )
        path_mock.return_value = configpath_exists

        self.assertRaises(RuntimeError, self.parser.parse_args, ["socket"])


class IgnoreConfigParserTestCase(unittest.TestCase):
    def test_unkown_config_file(self):
        test_config_path = __here__ / "foo.cfg"

        self.assertFalse(test_config_path.is_file())

        self.parser = CliParser("TestParser", "test.log")

        args = self.parser.parse_args(
            ["--config", str(test_config_path), "socket"]
        )

        self.assertEqual(args.timeout, DEFAULT_TIMEOUT)
        self.assertEqual(args.gmp_password, "")
        self.assertEqual(args.gmp_username, "")
        self.assertEqual(args.socketpath, DEFAULT_UNIX_SOCKET_PATH)

    def test_unkown_config_file_in_unkown_dir(self):
        test_config_path = __here__ / "foo" / "foo.cfg"

        self.assertFalse(test_config_path.is_file())

        self.parser = CliParser("TestParser", "test.log")

        args = self.parser.parse_args(
            ["--config", str(test_config_path), "socket"]
        )

        self.assertEqual(args.timeout, DEFAULT_TIMEOUT)
        self.assertEqual(args.gmp_password, "")
        self.assertEqual(args.gmp_username, "")
        self.assertEqual(args.socketpath, DEFAULT_UNIX_SOCKET_PATH)


class ParserTestCase(unittest.TestCase):
    def setUp(self):
        self.parser = CliParser(
            "TestParser", "test.log", ignore_config=True, prog="gvm-test-cli"
        )


class RootArgumentsParserTest(ParserTestCase):
    def test_config(self):
        args = self.parser.parse_args(["--config", "foo.cfg", "socket"])
        self.assertEqual(args.config, "foo.cfg")

    def test_defaults(self):
        args = self.parser.parse_args(["socket"])
        self.assertEqual(args.config, "~/.config/gvm-tools.conf")
        self.assertEqual(args.gmp_password, "")
        self.assertEqual(args.gmp_username, "")
        self.assertEqual(args.timeout, 60)
        self.assertIsNone(args.loglevel)

    def test_loglevel(self):
        args = self.parser.parse_args(["--log", "ERROR", "socket"])
        self.assertEqual(args.loglevel, "ERROR")

    def test_loglevel_after_subparser(self):
        with SuppressOutput(suppress_stderr=True):
            with self.assertRaises(SystemExit):
                self.parser.parse_args(["socket", "--log", "ERROR"])

    def test_timeout(self):
        args = self.parser.parse_args(["--timeout", "1000", "socket"])
        self.assertEqual(args.timeout, 1000)

    def test_timeout_after_subparser(self):
        with SuppressOutput(suppress_stderr=True):
            with self.assertRaises(SystemExit):
                self.parser.parse_args(["socket", "--timeout", "1000"])

    def test_gmp_username(self):
        args = self.parser.parse_args(["--gmp-username", "foo", "socket"])
        self.assertEqual(args.gmp_username, "foo")

    def test_gmp_username_after_subparser(self):
        with SuppressOutput(suppress_stderr=True):
            with self.assertRaises(SystemExit):
                self.parser.parse_args(["socket", "--gmp-username", "foo"])

    def test_gmp_password(self):
        args = self.parser.parse_args(["--gmp-password", "foo", "socket"])
        self.assertEqual(args.gmp_password, "foo")

    def test_gmp_password_after_subparser(self):
        with SuppressOutput(suppress_stderr=True):
            with self.assertRaises(SystemExit):
                self.parser.parse_args(["socket", "--gmp-password", "foo"])

    def test_with_unknown_args(self):
        args, script_args = self.parser.parse_known_args(
            ["--gmp-password", "foo", "socket", "--bar", "--bar2"]
        )
        self.assertEqual(args.gmp_password, "foo")
        self.assertEqual(script_args, ["--bar", "--bar2"])

    @patch("gvmtools.parser.logging")
    def test_socket_has_no_timeout(self, _logging_mock):
        # pylint: disable=protected-access
        args_mock = unittest.mock.MagicMock()
        args_mock.timeout = -1
        self.parser._parser.parse_known_args = unittest.mock.MagicMock(
            return_value=(args_mock, unittest.mock.MagicMock())
        )

        args, _ = self.parser.parse_known_args(
            ["socket", "--timeout", "--", "-1"]
        )

        self.assertIsNone(args.timeout)

    @patch("gvmtools.parser.logging")
    @patch("gvmtools.parser.argparse.ArgumentParser.print_usage")
    @patch("gvmtools.parser.argparse.ArgumentParser._print_message")
    def test_no_args_provided(
        self, _logging_mock, _print_usage_mock, _print_message
    ):
        # pylint: disable=protected-access
        self.parser._set_defaults = unittest.mock.MagicMock()

        self.assertRaises(SystemExit, self.parser.parse_known_args, None)


class SocketParserTestCase(ParserTestCase):
    def test_defaults(self):
        args = self.parser.parse_args(["socket"])
        self.assertEqual(args.socketpath, DEFAULT_UNIX_SOCKET_PATH)

    def test_connection_type(self):
        args = self.parser.parse_args(["socket"])
        self.assertEqual(args.connection_type, "socket")

    def test_sockpath(self):
        args = self.parser.parse_args(["socket", "--sockpath", "foo.sock"])
        self.assertEqual(args.socketpath, "foo.sock")

    def test_socketpath(self):
        args = self.parser.parse_args(["socket", "--socketpath", "foo.sock"])
        self.assertEqual(args.socketpath, "foo.sock")


class SshParserTestCase(ParserTestCase):
    def test_defaults(self):
        args = self.parser.parse_args(["ssh", "--hostname=foo"])
        self.assertEqual(args.port, 22)
        self.assertEqual(args.ssh_username, "gmp")
        self.assertEqual(args.ssh_password, "gmp")

    def test_connection_type(self):
        args = self.parser.parse_args(["ssh", "--hostname=foo"])
        self.assertEqual(args.connection_type, "ssh")

    def test_hostname(self):
        args = self.parser.parse_args(["ssh", "--hostname", "foo"])
        self.assertEqual(args.hostname, "foo")

    def test_port(self):
        args = self.parser.parse_args(
            ["ssh", "--hostname", "foo", "--port", "123"]
        )
        self.assertEqual(args.port, 123)

    def test_ssh_username(self):
        args = self.parser.parse_args(
            ["ssh", "--hostname", "foo", "--ssh-username", "foo"]
        )
        self.assertEqual(args.ssh_username, "foo")

    def test_ssh_password(self):
        args = self.parser.parse_args(
            ["ssh", "--hostname", "foo", "--ssh-password", "foo"]
        )
        self.assertEqual(args.ssh_password, "foo")


class TlsParserTestCase(ParserTestCase):
    def test_defaults(self):
        args = self.parser.parse_args(["tls", "--hostname=foo"])
        self.assertIsNone(args.certfile)
        self.assertIsNone(args.keyfile)
        self.assertIsNone(args.cafile)
        self.assertEqual(args.port, 9390)

    def test_connection_type(self):
        args = self.parser.parse_args(["tls", "--hostname=foo"])
        self.assertEqual(args.connection_type, "tls")

    def test_hostname(self):
        args = self.parser.parse_args(["tls", "--hostname", "foo"])
        self.assertEqual(args.hostname, "foo")

    def test_port(self):
        args = self.parser.parse_args(
            ["tls", "--hostname", "foo", "--port", "123"]
        )
        self.assertEqual(args.port, 123)

    def test_certfile(self):
        args = self.parser.parse_args(
            ["tls", "--hostname", "foo", "--certfile", "foo.cert"]
        )
        self.assertEqual(args.certfile, "foo.cert")

    def test_keyfile(self):
        args = self.parser.parse_args(
            ["tls", "--hostname", "foo", "--keyfile", "foo.key"]
        )
        self.assertEqual(args.keyfile, "foo.key")

    def test_cafile(self):
        args = self.parser.parse_args(
            ["tls", "--hostname", "foo", "--cafile", "foo.ca"]
        )
        self.assertEqual(args.cafile, "foo.ca")

    def test_no_credentials(self):
        args = self.parser.parse_args(
            ["tls", "--hostname", "foo", "--no-credentials"]
        )
        self.assertTrue(args.no_credentials)


class CustomizeParserTestCase(ParserTestCase):
    def test_add_optional_argument(self):
        self.parser.add_argument("--foo", type=int)

        args = self.parser.parse_args(["socket", "--foo", "123"])
        self.assertEqual(args.foo, 123)

        args = self.parser.parse_args(
            ["ssh", "--hostname", "bar", "--foo", "123"]
        )
        self.assertEqual(args.foo, 123)

        args = self.parser.parse_args(
            ["tls", "--hostname", "bar", "--foo", "123"]
        )
        self.assertEqual(args.foo, 123)

    def test_add_positional_argument(self):
        self.parser.add_argument("foo", type=int)
        args = self.parser.parse_args(["socket", "123"])

        self.assertEqual(args.foo, 123)

    def test_add_protocol_argument(self):
        self.parser.add_protocol_argument()

        args = self.parser.parse_args(["socket"])
        self.assertEqual(args.protocol, "GMP")

        args = self.parser.parse_args(["--protocol", "OSP", "socket"])

        self.assertEqual(args.protocol, "OSP")


class HelpFormattingParserTestCase(ParserTestCase):
    # pylint: disable=protected-access
    maxDiff = None
    python_version = ".".join([str(i) for i in sys.version_info[:2]])

    def setUp(self):
        super().setUp()

        # ensure all tests are using the same terminal width
        self.columns = os.environ.get("COLUMNS")
        os.environ["COLUMNS"] = "80"

    def tearDown(self):
        super().tearDown()

        if not self.columns:
            del os.environ["COLUMNS"]
        else:
            os.environ["COLUMNS"] = self.columns

    def _snapshot_specific_path(self, name):
        return __here__ / f"{name}.{self.python_version}.snap"

    def _snapshot_generic_path(self, name):
        return __here__ / f"{name}.snap"

    def _snapshot_failed_path(self, name):
        return __here__ / f"{name}.{self.python_version}-failed.snap"

    def _snapshot_path(self, name):
        snapshot_specific_path = self._snapshot_specific_path(name)

        if snapshot_specific_path.exists():
            return snapshot_specific_path

        return self._snapshot_generic_path(name)

    def assert_snapshot(self, name, output):
        path = self._snapshot_path(name)

        if not path.exists():
            path.write_text(output)

        content = path.read_text(encoding="utf-8")

        try:
            self.assertEqual(content, output, "Snapshot differs from output")
        except AssertionError:
            # write new output to snapshot file
            # reraise error afterwards
            path = self._snapshot_failed_path(name)
            path.write_text(output)
            raise

    @unittest.skipIf(
        sys.version_info[:2] > (3, 10), "missing snapshot for Python 3.11"
    )
    def test_root_help(self):
        help_output = self.parser._parser.format_help()
        self.assert_snapshot("root_help", help_output)

    @unittest.skipIf(
        sys.version_info[:2] > (3, 10), "missing snapshot for Python 3.11"
    )
    def test_socket_help(self):
        help_output = self.parser._parser_socket.format_help()
        self.assert_snapshot("socket_help", help_output)

    @unittest.skipIf(
        sys.version_info[:2] > (3, 10), "missing snapshot for Python 3.11"
    )
    def test_ssh_help(self):
        self.parser._set_defaults(None)
        help_output = self.parser._parser_ssh.format_help()
        self.assert_snapshot("ssh_help", help_output)

    @unittest.skipIf(
        sys.version_info[:2] > (3, 10), "missing snapshot for Python 3.11"
    )
    def test_tls_help(self):
        self.parser._set_defaults(None)
        help_output = self.parser._parser_tls.format_help()
        self.assert_snapshot("tls_help", help_output)


class CreateParserFunctionTestCase(unittest.TestCase):
    # pylint: disable=protected-access
    def test_create_parser(self):
        description = "parser description"
        logfilename = "logfilename"

        parser = create_parser(description, logfilename)

        self.assertIsInstance(parser, CliParser)
        self.assertEqual(parser._logfilename, logfilename)
        self.assertEqual(parser._bootstrap_parser.description, description)


class CreateConnectionTestCase(unittest.TestCase):
    def test_create_unix_socket_connection(self):
        self.perform_create_connection_test()

    def test_create_tls_connection(self):
        self.perform_create_connection_test("tls", TLSConnection)

    def test_create_ssh_connection(self):
        self.perform_create_connection_test("ssh", SSHConnection, 22)

    def perform_create_connection_test(
        self,
        connection_type="socket",
        connection_class=UnixSocketConnection,
        port=None,
    ):
        connection = create_connection(connection_type, port=port)
        self.assertIsInstance(connection, connection_class)



# -*- coding: utf-8 -*-
# SPDX-FileCopyrightText: 2020-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see<http://www.gnu.org/licenses/>


import unittest
from pathlib import Path
from unittest.mock import MagicMock, patch

from lxml import etree

from . import GmpMockFactory, load_script

CWD = Path(__file__).absolute().parent


class SendTasksTestCase(unittest.TestCase):
    def setUp(self):
        self.send_tasks = load_script(
            (CWD.parent.parent / "scripts"), "send-tasks"
        )

    @patch("builtins.input", lambda *args: "y")
    @patch("gvm.protocols.latest.Gmp", new_callable=GmpMockFactory)
    def test_sent_task(self, mock_gmp: GmpMockFactory):
        task_xml_path = CWD / "example_task.xml"
        task_xml_str = task_xml_path.read_text(encoding="utf-8")

        self.send_tasks.numerical_option = MagicMock(return_value=1)

        configs_file = CWD / "get_scan_configs.xml"
        configs = configs_file.read_text(encoding="utf-8")
        mock_gmp.mock_response("get_scan_configs", configs)

        mock_gmp.mock_response(
            "get_scanners",
            '<get_scanners_response status="200" status_text="OK">'
            '<scanner id="c1c85af7-0cca-4690-8ccc-c79feb5588cf">'
            "<name>as</name>"
            "</scanner>"
            '<scanner id="6acd0832-df90-11e4-b9d5-28d24461215b">'
            "<name>CVE</name>"
            "</scanner>"
            '<scanner id="08b69003-5fc2-4037-a479-93b440211c73">'
            "<name>OpenVAS Default</name>"
            "</scanner>"
            "</get_scanners_response>",
        )

        mock_gmp.mock_response(
            "get_targets",
            '<get_targets_response status="200" status_text="OK">'
            '<target id="60f95d0e-029e-4931-a13a-b1d11260517d">'
            "<name>own</name>"
            "</target>"
            '<target id="ead9576c-5a4d-4081-b98d-ccd77d5d16f8">'
            "<name>Target for xn</name>"
            "</target>"
            '<target id="6c9f73f5-f14c-42bf-ab44-edb8d2493dbc">'
            "<name>Unnamed</name>"
            "</target>"
            '<target id="a1f478c1-27d0-4d8c-959f-150625186421">'
            "<name>work</name>"
            "</target>"
            '<target id="5ca97fe1-694d-4e4a-bd4c-55529719d17e">'
            "<name>work2</name>"
            "</target>"
            "</get_targets_response>",
        )

        mock_gmp.mock_response(
            "create_task",
            '<create_task_response status="201" status_text="OK,'
            'resource created" id="c8ef0597-e2c1-4e23-869f-072fa2914bf2"/>',
        )

        task = etree.XML(task_xml_str)

        tasks = self.send_tasks.parse_send_xml_tree(mock_gmp.gmp_protocol, task)
        self.assertEqual(tasks, ["c8ef0597-e2c1-4e23-869f-072fa2914bf2"])



#!/usr/bin/env python3
# coding: utf-8
# -
# Copyright © 2015, 2017, 2020, 2022
#       mirabilos <t.glaser@tarent.de>
# Licensor: tarent solutions GmbH
#
# Provided that these terms and disclaimer and all copyright notices
# are retained or reproduced in an accompanying document, permission
# is granted to deal in this work without restriction, including un‐
# limited rights to use, publicly perform, distribute, sell, modify,
# merge, give away, or sublicence.
#
# This work is provided “AS IS” and WITHOUT WARRANTY of any kind, to
# the utmost extent permitted by applicable law, neither express nor
# implied; without malicious intent or gross negligence. In no event
# may a licensor, author or contributor be held liable for indirect,
# direct, other damage, loss, or other issues arising in any way out
# of dealing in the work, even if advised of the possibility of such
# damage or existence of a defect, except proven that it results out
# of said person’s immediate fault when using the work as intended.

r"""SSV reader/writer and CSV writer library

This module offers the following classes:

- CSVInvalidCharacterError, CSVShapeError -- Exception classes
  that can be thrown by code from this library

- CSVPrinter -- configurable CSV row formatter and writer that
  ensures the output is quoted properly and in rectangular shape

- SSVPrinter -- CSVPrinter configured to produce SSV output

- CSVWriter, SSVWriter -- same but writing to a file-like object

- SSVReader -- class to read SSV files, returning lists of str|bytes
  (depending on the input file binary flag)

When run directly, it acts as SSV to CSV converter, which may be of
limited use but demonstrates how to use the module somewhat; -h for
usage (help).
"""

__all__ = [
    "CSVInvalidCharacterError",
    "CSVPrinter",
    "CSVShapeError",
    "CSVWriter",
    "SSVPrinter",
    "SSVReader",
    "SSVWriter",
]

import re
import sys
from typing import IO, AnyStr, List, Optional, TextIO


class CSVShapeError(Exception):
    r"""Error: data to write did not have a consistent amount of columns.

    The message string is descriptive, but the want and got fields of
    an object may be user-accessed.
    """

    def __init__(self, want: int, got: int) -> None:
        Exception.__init__(
            self, f"got {got} column{got != 1 and 's' or ''} but wanted {want}"
        )
        self.want = want  # type: int
        self.got = got  # type: int


class CSVInvalidCharacterError(Exception):
    r"""Error: disallowed characters in cell (writer) / row (reader).

    The message is deliberately constant in order to not show the actual
    cell or row content in logs, etc. (in case it’s a password) but the
    actual content is available in the questionable_content field.
    """

    def __init__(
        self, value: AnyStr, what: str = "prohibited character in cell"
    ) -> None:
        Exception.__init__(self, what)
        self.questionable_content = value  # type: Union[str, bytes]


class CSVPrinter(object):
    r"""CSV writer library, configurable.

    The defaults follow RFC 4180 and thus are suitable for use with most
    environments; newlines embedded in cell data are normalised (from
    ASCII/Unix/Mac to ASCII) by default, every cell data is quoted.

    The following arguments configure the writer instance:
    - sep -- output cell separator: ',' (default) or ';' or '\t'
    - quot -- output quote character (default '"'), escape by doubling;
        None to disable quoting and disallow embedded newlines (but not
        double quotes; the caller must not pass any if the result needs
        to conform to the RFC or just use default quoting of course)
    - eol -- output line terminator: '\r\n' or (Unix) '\n' or (Mac) '\r'
    - qnl -- output embedded newline, should match eol (default '\r\n');
        None to disable embedded newline normalisation

    These arguments must all be str, bytes is not supported. Cell data
    passed in that is not str will be stringified. Rows are written or
    returned as str; however, ASCII or a compatible encoding needs to
    be used (for conformance and portability); UTF-8 is ideal.

    Note: RFC 4180 permits only printable ASCII and space and, if quoted,
    newlines in cell data but this library permits any character except
    NUL, (unquoted) CR and LF.
    """

    # embedded NUL is never permitted
    _invf = re.compile("[\x00]")  # type: Pattern[str]
    # normalise embedded newlines (match)
    _nlf = re.compile("\r\n?|(?<!\r)\n")  # type: Pattern[str]
    # count to ensure rectangular shape of output
    _ncols = -1  # type: int

    # default line ending is ASCII (“DOS”)
    def __init__(
        self,
        sep: str = ",",
        quot: Optional[str] = '"',
        eol: str = "\r\n",
        qnl: Optional[str] = "\r\n",
    ) -> None:
        if quot is None:
            # cell joiner ('","')
            self._sep = sep  # type: str
            # one quote, e.g. for line beginning/end
            self._quots = ""  # type: str
            # two quotes to escape
            self._quotd = ""  # type: str
            # forbid newlines if we cannot quote them
            self._invf = re.compile("[\x00\r\n]")
        else:
            self._sep = quot + sep + quot
            self._quots = quot
            self._quotd = quot + quot
        # None if not quoting
        self._quot = quot  # type: Optional[str]
        # None or embedded newline replacement string
        self._nlrpl = qnl  # type: Optional[str]
        # EOL string
        self._eol = eol  # type: str

    def _mapcell(self, cell) -> str:
        if isinstance(cell, str):
            cstr = cell  # type: str
        else:
            cstr = str(cell)
        if self._invf.search(cstr) is not None:
            raise CSVInvalidCharacterError(cstr)
        if self._nlrpl is not None:
            cstr = self._nlf.sub(self._nlrpl, cstr)
        if self._quot is not None:
            cstr = cstr.replace(self._quots, self._quotd)
        return cstr

    def write(self, *args) -> None:
        r"""Print a CSV line (row) to standard output.

        - *args -- cell data by columns

        Note: reconfigures the newline mode of sys.stdout once,
        but make sure to run sys.stdout.reconfigure(newline='\n')
        e.g. when emitting an MS Excel sep= line and get the line
        ending for that right; see _main() for an example.
        """
        if hasattr(sys.stdout, "reconfigure"):
            sys.stdout.reconfigure(newline="\n")  # type: ignore
        setattr(CSVPrinter, "write", getattr(CSVPrinter, "_write"))
        delattr(CSVPrinter, "_write")
        return self.write(*args)

    def _write(self, *args) -> None:
        print(self.format(*args), end="")

    _write.__doc__ = write.__doc__

    def format(self, *args) -> str:
        r"""Produce a CSV row from cells.

        - *args -- cell data by columns

        Returns the row, including the trailing newline, as string.
        """
        if self._ncols == -1:
            self._ncols = len(args)
        elif self._ncols != len(args):
            raise CSVShapeError(self._ncols, len(args))
        cells = map(self._mapcell, args)
        return self._quots + self._sep.join(cells) + self._quots + self._eol


class CSVWriter(CSVPrinter):
    r"""CSV writer library, configurable.

    The defaults follow RFC 4180 and thus are suitable for use with most
    environments; newlines embedded in cell data are normalised (from
    ASCII/Unix/Mac to ASCII) by default, every cell data is quoted.

    The following arguments configure the writer instance:
    - file -- file-like object to output CSV to
    - sep -- output cell separator: ',' (default) or ';' or '\t'
    - quot -- output quote character (default '"'), escape by doubling;
        None to disable quoting and disallow embedded newlines (but not
        double quotes; the caller must not pass any if the result needs
        to conform to the RFC or just use default quoting of course)
    - eol -- output line terminator: '\r\n' or (Unix) '\n' or (Mac) '\r'
    - qnl -- output embedded newline, should match eol (default '\r\n');
        None to disable embedded newline normalisation

    These arguments must all be str, bytes is not supported. Cell data
    passed in that is not str will be stringified. Rows are written or
    returned as str; however, ASCII or a compatible encoding needs to
    be used (for conformance and portability); UTF-8 is ideal.

    Note: RFC 4180 permits only printable ASCII and space and, if quoted,
    newlines in cell data but this library permits any character except
    NUL, (unquoted) CR and LF.

    Note: the file argument will be reconfigured to disable automatic
    '\n' conversion; if prepending data (e.g. a sep= line for MS Excel)
    use the writeln() method.
    """

    def __init__(
        self,
        file: TextIO,
        sep: str = ",",
        quot: Optional[str] = '"',
        eol: str = "\r\n",
        qnl: Optional[str] = "\r\n",
    ) -> None:
        CSVPrinter.__init__(self, sep, quot, eol, qnl)
        # disable any automatic newline conversion if preset
        file.reconfigure(newline="\n")  # type: ignore
        self.outfile = file

    def write(self, *args) -> None:
        r"""Print a CSV line (row) to the output file.

        - *args -- cell data by columns
        """
        print(self.format(*args), end="", file=self.outfile)

    def writeln(self, line: str) -> None:
        r"""Print an arbitrary nōn-CSV line to the output file.

        - line -- str to output; trailing newline is automatically added
        """
        print(line, end=self._eol, file=self.outfile)


class SSVPrinter(CSVPrinter):
    r"""SSV writer library.

    This subclass sets up a CSVPrinter instance to produce SSV (see below).
    The writer supports str, or stringified arguments, only, not bytes.
    The caller must ensure the encoding is UTF-8 (ideally), or at least
    ASCII-compatible (CR, LF and \x1F require identity mapping), and that
    CR or LF characters output are not converted.

    shell-parseable separated values (or separator-separated values)
    is an idea to make CSV into something usable:

    • newline (\x0A) is row separator
    • unit separator (\x1F) is column separator
    • n̲o̲ quotes or escape characters
    • carriage return (\x0D) represents embedded newlines in cells

    Cell content is, in theory, arbitrary binary except NUL and
    the separators (\x1F and \x0A). In practice it should be UTF-8.

    SSV can be easily read from shell scripts:

        while IFS=$'\x1F' read -r col1 col2…; do
            # do something
        done
    """

    def __init__(self) -> None:
        CSVPrinter.__init__(self, sep="\x1F", quot=None, eol="\n", qnl="\r")
        # not permitted in SSV data
        self._invf = re.compile("[\x00\x1F]")


class SSVWriter(CSVWriter):
    r"""SSV writer library (same as SSVPrinter except to file)"""

    def __init__(self, file: TextIO) -> None:
        # pylint: disable=C0301
        CSVWriter.__init__(
            self, file, sep="\x1F", quot=None, eol="\n", qnl="\r"
        )
        # not permitted in SSV data
        self._invf = re.compile("[\x00\x1F]")


if SSVPrinter.__doc__ is not None:
    SSVWriter.__doc__ = SSVPrinter.__doc__.replace("CSVPrinter", "CSVWriter")


class SSVReader(object):
    r"""SSV reader library.

    This library is initialised with a files-like object that must
    support .readline() and either must not use newline conversion
    or support .reconfigure() as in _io.TextIOWrapper, which is
    called with newline='\n' if it exists. SSVReader.read() will
    then proceed to read from it.

    See SSVPrinter about the SSV format.
    """

    def __init__(self, file: IO) -> None:
        if hasattr(file, "reconfigure"):
            # see https://bugs.python.org/issue46695 though
            file.reconfigure(newline="\n")  # type: ignore
        self.f = file  # type: IO

    @staticmethod
    def _read(
        line: AnyStr,
        lf: AnyStr,
        cr: AnyStr,
        us: AnyStr,
        nl: AnyStr,
        nul: AnyStr,
    ) -> List[AnyStr]:
        if line.find(nul) != -1:
            raise CSVInvalidCharacterError(line, "NUL in row")
        if line[-1:] != lf:
            raise CSVInvalidCharacterError(line, "unterminated row")
        line = line[:-1]
        if line.find(lf) != -1:
            raise CSVInvalidCharacterError(line, "LF in row")
        return line.replace(cr, nl).split(us)

    def read(self) -> Optional[List[AnyStr]]:
        r"""Read and decode one SSV line.

        Returns a list of cells, or None on EOF.
        """
        line = self.f.readline()  # type: Optional[AnyStr]
        if not line:
            return None
        if isinstance(line, str):
            return self._read(line, "\n", "\r", "\x1F", "\r\n", "\x00")
        if isinstance(line, bytes):
            return self._read(line, b"\n", b"\r", b"\x1F", b"\r\n", b"\x00")
        raise TypeError()


# mostly example of how to use this
def _main() -> None:
    # pylint: disable=C0103
    newline_ways = {
        "ascii": "\r\n",
        "unix": "\n",
        "mac": "\r",
    }
    p = argparse.ArgumentParser(
        description="Converts SSV to CSV.",
        # part of https://bugs.python.org/issue46700 workaround
        add_help=False,
    )
    g = p.add_argument_group("Options")  # issue46700
    g.add_argument("-h", action="help", help="show this help message and exit")
    g.add_argument(
        "-s",
        metavar="sep",
        help="cell separator, e.g. \x27,\x27 (default: tab)",
        default="\t",
    )
    g.add_argument(
        "-q",
        metavar="qch",
        help="quote character, e.g. \x27\x22\x27 (default: none)",
        default=None,
    )
    g.add_argument(
        "-n",
        metavar="eoltype",
        choices=list(newline_ways.keys()),
        help="line endings (ascii (default), unix, mac)",
        default="ascii",
    )
    g.add_argument(
        "-P",
        metavar="preset",
        choices=["std", "sep", "ssv"],
        help="predefined config (std=RFC 4180, sep=Excel header, ssv=SSV)",
    )
    g = p.add_argument_group("Arguments")  # issue46700
    g.add_argument(
        "file",
        nargs="?",
        help='SSV file to read, "-" for stdin (default)',
        default="-",
    )
    args = p.parse_args()
    if args.P in ("std", "sep"):
        args.s = ","
        args.q = '"'
        args.n = "ascii"
    nl = newline_ways[args.n]
    if hasattr(sys.stdout, "reconfigure"):
        sys.stdout.reconfigure(newline="\n")  # type: ignore
    if args.P == "sep":
        print(f"sep={args.s}", end=nl)
    if args.P != "ssv":
        w = CSVPrinter(args.s, args.q, nl, nl)
    else:
        w = SSVPrinter()

    def _convert(f):
        r = SSVReader(f)
        # no walrus in Python 3.7 yet ☹
        while True:
            row = r.read()
            if row is None:
                break
            w.write(*row)

    if args.file == "-":
        _convert(sys.stdin)
    else:
        with open(args.file, "r", encoding="utf-8") as file:
            _convert(file)


if __name__ == "__main__":
    import argparse

    _main()



# SPDX-FileCopyrightText: 2024 Martin Boller
#
# SPDX-License-Identifier: GPL-3.0-or-later

from argparse import Namespace

from gvm.protocols.gmp import Gmp


def stop_tasks(gmp: Gmp) -> None:
    tasks = gmp.get_tasks(
        filter_string="rows=-1 status=Running or status=Requested or status=Queued"
    )
    try:
        for task_id in tasks.xpath("task/@id"):
            print(f"Stopping task {task_id} ... ")
            status_text = gmp.stop_task(task_id).xpath("@status_text")[0]
            print(status_text)
    except Exception as e:
        print(f"{e=}")


def main(gmp: Gmp, args: Namespace) -> None:
    # pylint: disable=undefined-variable
    print("This script stops all tasks on the system.\n")

    stop_tasks(gmp)


if __name__ == "__gmp__":
    main(gmp, args)



# SPDX-FileCopyrightText: 2024 Martin Boller
#
# SPDX-License-Identifier: GPL-3.0-or-later

from argparse import Namespace

from gvm.protocols.gmp import Gmp
from gvmtools.helper import Table


def main(gmp: Gmp, args: Namespace) -> None:
    # pylint: disable=unused-argument

    response_xml = gmp.get_filters(filter_string="rows=-1")
    filters_xml = response_xml.xpath("filter")

    heading = ["#", "Name", "Id", "Modified", "Type", "Term"]

    rows = []
    numberRows = 0

    print("Listing filters.\n")

    for filter in filters_xml:
        # Count number of reports
        numberRows = numberRows + 1
        # Cast/convert to text to show in list
        rowNumber = str(numberRows)

        name = "".join(filter.xpath("name/text()"))
        modified = "".join(filter.xpath("modification_time/text()"))
        term = "".join(filter.xpath("term/text()"))
        type = "".join(filter.xpath("type/text()"))
        filter_id = filter.get("id")
        rows.append([rowNumber, name, filter_id, modified, type, term])

    print(Table(heading=heading, rows=rows))


if __name__ == "__gmp__":
    main(gmp, args)



# -*- coding: utf-8 -*-
# SPDX-FileCopyrightText: 2020-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see<http://www.gnu.org/licenses/>


import unittest
from pathlib import Path
from unittest.mock import patch

from . import GmpMockFactory, load_script

CWD = Path(__file__).absolute().parent


class StartAlertScanTestCase(unittest.TestCase):
    def setUp(self):
        self.start_alert_scan = load_script(
            (CWD.parent.parent / "scripts"), "start-alert-scan"
        )

    @patch("gvm.protocols.latest.Gmp", new_callable=GmpMockFactory)
    def test_get_scan_config(self, mock_gmp: GmpMockFactory):
        configs_file = CWD / "get_scan_configs.xml"
        configs = configs_file.read_text(encoding="utf-8")
        mock_gmp.mock_response("get_scan_configs", configs)

        # Full and Fast
        config_id = self.start_alert_scan.get_scan_config(
            gmp=mock_gmp.gmp_protocol, config=0
        )
        self.assertEqual(config_id, "daba56c8-73ec-11df-a475-002264764cea")

        # Full and Fast ultimate
        config_id = self.start_alert_scan.get_scan_config(
            gmp=mock_gmp.gmp_protocol, config=1
        )
        self.assertEqual(config_id, "698f691e-7489-11df-9d8c-002264764cea")

        # Full and Fast deep
        config_id = self.start_alert_scan.get_scan_config(
            gmp=mock_gmp.gmp_protocol, config=2
        )
        self.assertEqual(config_id, "708f25c4-7489-11df-8094-002264764cea")

        # Full and Fast deep ultimate
        config_id = self.start_alert_scan.get_scan_config(
            gmp=mock_gmp.gmp_protocol, config=3
        )
        self.assertEqual(config_id, "74db13d6-7489-11df-91b9-002264764cea")

        # System Discovery
        config_id = self.start_alert_scan.get_scan_config(
            gmp=mock_gmp.gmp_protocol, config=4
        )
        self.assertEqual(config_id, "bbca7412-a950-11e3-9109-406186ea4fc5")

        with self.assertRaises(ValueError):
            config_id = self.start_alert_scan.get_scan_config(
                gmp=mock_gmp.gmp_protocol, config=-1
            )

    @patch("gvm.protocols.latest.Gmp", new_callable=GmpMockFactory)
    def test_get_alert(self, mock_gmp: GmpMockFactory):
        sender_email = "sender@test.com"
        recipient_email = "recipient@test.com"
        alert_name = "test_alert"
        alert_id = "3eefd4b9-59ec-48d6-b84d-f6a73bdb909f"

        alerts_file = CWD / "get_alerts.xml"
        alerts = alerts_file.read_text(encoding="utf-8")
        mock_gmp.mock_response("get_alerts", alerts)
        mock_gmp.mock_response(
            "create_alert",
            '<create_alert_response status="201" status_text='
            f'"OK, resource created" id="{alert_id}"/>',
        )

        returned_id = self.start_alert_scan.get_alert(
            gmp=mock_gmp.gmp_protocol,
            alert_name=alert_name,
            recipient_email=recipient_email,
            sender_email=sender_email,
        )

        self.assertEqual(alert_id, returned_id)

    @patch("gvm.protocols.latest.Gmp", new_callable=GmpMockFactory)
    def test_get_target(self, mock_gmp: GmpMockFactory):
        target_name = "test_target"
        hosts = ["127.0.0.1", "8.8.8.8"]
        ports = "T:1-3,5,7,9,11,13,17-25"
        port_list_name = "test_port_list"

        port_list_id = "6742e61a-a7b0-45dd-a8e1-35751c970958"
        target_id = "3b76a0c2-14fc-4de2-868c-35132977a25e"

        mock_gmp.mock_response(
            "create_port_list",
            '<create_port_list_response status="201" status_text='
            f'"OK, resource created" id="{port_list_id}"/>',
        )

        mock_gmp.mock_response(
            "create_target",
            '<create_target_response status="201" status_text='
            f'"OK, resource created" id="{target_id}"/>',
        )

        returned_id = self.start_alert_scan.get_target(
            gmp=mock_gmp.gmp_protocol,
            target_name=target_name,
            hosts=hosts,
            ports=ports,
            port_list_name=port_list_name,
        )

        self.assertEqual(target_id, returned_id)

    @patch("gvm.protocols.latest.Gmp", new_callable=GmpMockFactory)
    def test_create_and_start_task(self, mock_gmp: GmpMockFactory):
        alert_name = "test_alert"
        alert_id = "3eefd4b9-59ec-48d6-b84d-f6a73bdb909f"
        target_id = "3b76a0c2-14fc-4de2-868c-35132977a25e"
        config_id = "daba56c8-73ec-11df-a475-002264764cea"
        scanner_id = "08b69003-5fc2-4037-a479-93b440211c73"

        task_id = "d78453ab-d907-44b6-abe0-2ef54a77f1c2"

        mock_gmp.mock_response(
            "create_task",
            '<create_task_response status="201" status_text='
            f'"OK, resource created" id="{task_id}"/>',
        )

        mock_gmp.mock_response(
            "get_tasks",
            """
<get_tasks_response status="200" status_text="OK">
  <apply_overrides>0</apply_overrides>
  <filters id="">
    <term>
        apply_overrides=0 min_qod=70
        name="Alert Scan for Alert test_alert" first=1 rows=100 sort=name
    </term>
    <keywords>
      <keyword>
        <column>apply_overrides</column>
        <relation>=</relation>
        <value>0</value>
      </keyword>
      <keyword>
        <column>min_qod</column>
        <relation>=</relation>
        <value>70</value>
      </keyword>
      <keyword>
        <column>name</column>
        <relation>=</relation>
        <value>"Alert Scan for Alert test_alert"</value>
      </keyword>
      <keyword>
        <column>first</column>
        <relation>=</relation>
        <value>1</value>
      </keyword>
      <keyword>
        <column>rows</column>
        <relation>=</relation>
        <value>100</value>
      </keyword>
      <keyword>
        <column>sort</column>
        <relation>=</relation>
        <value>name</value>
      </keyword>
    </keywords>
  </filters>
  <sort>
    <field>name<order>ascending</order></field>
  </sort>
  <tasks start="1" max="100"/>
  <task_count>27<filtered>0</filtered><page>0</page></task_count>
</get_tasks_response>
            """,
        )

        task_name = f"Alert Scan for Alert {alert_name}"

        returned_name = self.start_alert_scan.create_and_start_task(
            gmp=mock_gmp.gmp_protocol,
            config_id=config_id,
            target_id=target_id,
            scanner_id=scanner_id,
            alert_id=alert_id,
            alert_name=alert_name,
        )

        self.assertEqual(task_name, returned_name)



# SPDX-FileCopyrightText: 2018-2021 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

import sys
from argparse import Namespace

from gvm.protocols.gmp import Gmp
from gvmtools.helper import create_xml_tree, yes_or_no
from lxml.etree import Element


def check_args(args: Namespace) -> None:
    len_args = len(args.script) - 1
    if len_args != 1:
        message = """
        This script pulls target data from an xml document and feeds it to \
    a desired GSM
        One parameter after the script name is required.

        1. <xml_doc>  -- .xml file containing targets

        Example:
            $ gvm-script --gmp-username name --gmp-password pass \
    ssh --hostname <gsm> scripts/send-targets.gmp.py example_file.xml
        """
        print(message)
        sys.exit()


def parse_send_xml_tree(gmp: Gmp, xml_tree: Element) -> None:
    credential_options = [
        "ssh_credential",
        "smb_credential",
        "esxi_credential",
        "snmp_credential",
    ]
    counter = 1

    for target in xml_tree.xpath("target"):
        keywords = {}  # {'make_unique': True}

        keywords["name"] = target.find("name").text

        keywords["hosts"] = target.find("hosts").text.split(",")

        exclude_hosts = target.find("exclude_hosts").text
        if exclude_hosts is not None:
            keywords["exclude_hosts"] = exclude_hosts.split(",")

        comment = target.find("comment").text
        if comment is not None:
            keywords["comment"] = comment

        credentials = gmp.get_credentials()[0].xpath("//credential/@id")

        for credential in credential_options:
            cred_id = target.find(credential).xpath("@id")[0]
            if cred_id == "":
                continue
            if cred_id not in credentials:
                response = yes_or_no(
                    f"\nThe credential '{credential}' for 'target {counter}' "
                    "could not be located...\nWould you like to continue?"
                )

                if response is False:
                    print("Terminating...\n")
                    sys.exit()
                else:
                    continue

            key = f"{credential}_id"
            keywords[key] = cred_id
            elem_path = target.find(credential)
            port = elem_path.find("port")
            if port is not None and port.text is not None:
                port_key = f"{credential}_port"
                keywords[port_key] = elem_path.find("port").text

        alive_test = gmp.types.AliveTest.from_string(
            target.find("alive_tests").text
        )

        if alive_test is not None:
            keywords["alive_test"] = alive_test

        reverse_lookup_only = target.find("reverse_lookup_only").text
        if reverse_lookup_only == "1":
            keywords["reverse_lookup_only"] = 1

        reverse_lookup_unify = target.find("reverse_lookup_unify").text
        if reverse_lookup_unify == "1":
            keywords["reverse_lookup_unify"] = 1

        port_range = target.find("port_range")
        if port_range is not None:
            keywords["port_range"] = port_range.text

        if target.xpath("port_list/@id") is not None:
            port_list = {}
            port_list = target.xpath("port_list/@id")[0]
            keywords["port_list_id"] = port_list

        print(keywords)

        gmp.create_target(**keywords)

        counter += 1


def main(gmp: Gmp, args: Namespace) -> None:
    # pylint: disable=undefined-variable

    check_args(args)

    xml_doc = args.script[1]

    print("\nSending targets...")

    xml_tree = create_xml_tree(xml_doc)
    parse_send_xml_tree(gmp, xml_tree)

    print("\n  Target(s) created!\n")


if __name__ == "__gmp__":
    main(gmp, args)  # pylint: disable=undefined-variable



# SPDX-FileCopyrightText: 2017-2021 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

import time
from argparse import ArgumentParser, FileType, Namespace, RawTextHelpFormatter
from typing import List

from gvm.protocols.gmp import Gmp
from gvmtools.helper import error_and_exit

HELP_TEXT = (
    "This script will update target hosts information for a desired task.\n"
    'The given task needs to have the status "new".\n\n'
    "Example for starting up the routine:\n"
    "    $ gvm-script --gmp-username name --gmp-password pass "
    "ssh --hostname <gsm> scripts/update-task-target.gmp.py +hf hosts_file.csv "
    "+t 303fa0a6-aa9b-43c4-bac0-66ae0b2d1698"
)


def parse_args(args: Namespace) -> Namespace:  # pylint: disable=unused-argument
    """Parsing args ..."""

    parser = ArgumentParser(
        prefix_chars="+",
        add_help=False,
        formatter_class=RawTextHelpFormatter,
        description=HELP_TEXT,
    )

    parser.add_argument(
        "+h",
        "++help",
        action="help",
        help="Show this help message and exit.",
    )

    parser.add_argument(
        "+t",
        "++task-id",
        type=str,
        required=True,
        dest="task_id",
        help="UUID of task to be modified",
    )

    hosts_args = parser.add_mutually_exclusive_group()

    hosts_args.add_argument(
        "+hl",
        "++host-list",
        nargs="+",
        type=str,
        dest="host_list",
        help="Use the given hosts (IPs) for the new target.",
    )

    hosts_args.add_argument(
        "+hf",
        "++host-file",
        type=FileType("r"),
        dest="host_file",
        help=".csv file containing desired target hosts separated by ','",
    )

    script_args, _ = parser.parse_known_args()
    return script_args


def load_host_file(filename) -> List[str]:
    host_list = list()

    try:
        for line in filename.readlines():
            host = line.split(",")[0]
            host = host.strip()
            if len(host) == 0:
                continue
            host_list.append(host)

    except IOError as e:
        error_and_exit(f"Failed to read host_file: {str(e)} (exit)")

    if len(host_list) == 0:
        error_and_exit("Host file is empty (exit)")

    return host_list


def copy_send_target(gmp, host_list, old_target_id):
    keywords = {"hosts": host_list}

    keywords["comment"] = (
        "This target was automatically "
        f'modified: {time.strftime("%Y/%m/%d-%H:%M:%S")}'
    )

    old_target = gmp.get_target(target_id=old_target_id)[0]

    objects = ("reverse_lookup_only", "reverse_lookup_unify", "name")
    for obj in objects:
        var = old_target.xpath(f"{obj}/text()")[0]
        if var == "0":
            var = ""
        keywords[f"{obj}"] = var

    port_list = {}
    port_list = old_target.xpath("port_list/@id")[0]
    keywords["port_list_id"] = port_list

    keywords["name"] += "_copy"  # the name must differ from existing names

    new_target_id = gmp.create_target(**keywords).xpath("@id")[0]

    print("\n  New target created!\n")
    print(f"Target_id:   {new_target_id}")
    print(f'Target_name: {keywords["name"]}\n')

    return new_target_id


def create_target_hosts(gmp: Gmp, host_file, task_id, old_target_id):
    new_target_id = copy_send_target(gmp, host_file, old_target_id)

    gmp.modify_task(task_id=task_id, target_id=new_target_id)

    print("  Task successfully modified!\n")


def check_to_delete(gmp: Gmp, target_id: str) -> None:
    target = gmp.get_target(target_id=target_id)[0]
    if "0" in target.xpath("in_use/text()"):
        gmp.delete_target(target_id=target_id)


def main(gmp: Gmp, args: Namespace) -> None:
    # pylint: disable=undefined-variable
    parsed_args = parse_args(args=args)

    host_list = parsed_args.host_list
    if parsed_args.host_file:
        host_list = load_host_file(filename=parsed_args.host_file)
    task_id = parsed_args.task_id

    task = gmp.get_task(task_id=task_id)[1]
    old_target_id = task.xpath("target/@id")[0]

    if old_target_id:
        create_target_hosts(gmp, host_list, task_id, old_target_id)
        check_to_delete(gmp, old_target_id)
    else:
        error_and_exit("The given task doesn't have an existing target.")


if __name__ == "__gmp__":
    main(gmp, args)



# SPDX-FileCopyrightText: 2017-2021 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

import sys
from argparse import Namespace
from random import choice, gauss

from gvm.protocols.gmp import Gmp
from gvmtools.helper import generate_random_ips


def check_args(args):
    len_args = len(args.script) - 1
    if len_args < 2:
        message = """
        This script generates random task data and feeds it to\
    a desired GSM
        It needs two parameters after the script name.

        1. <host_number> -- number of dummy hosts to select from
        2. <number>      -- number of targets to be generated

        In addition, if you would like for the number of targets generated
    to be randomized on a Gaussian distribution, add 'with-gauss'

        Example:
            $ gvm-script --gmp-username name --gmp-password pass \
    ssh --hostname <gsm> scripts/gen-random-targets.gmp.py 3 40 with-gauss
        """
        print(message)
        sys.exit()


def generate(gmp, args, n_targets, n_ips):
    ips = generate_random_ips(n_ips)

    if "with-gauss" in args.script:
        n_targets = int(gauss(n_targets, 2))

    for i in range(n_targets):
        host_ip = choice(ips)
        index = f"{{0:0>{len(str(n_targets))}}}"
        name = f"Target_{index.format(i + 1)}"

        gmp.create_target(name=name, make_unique=True, hosts=[host_ip])


def main(gmp: Gmp, args: Namespace) -> None:
    # pylint: disable=undefined-variable

    check_args(args)

    host_number = int(args.script[1])
    number_targets = int(args.script[2])

    print("Generating random targets...")

    generate(gmp, args, number_targets, host_number)


if __name__ == "__gmp__":
    main(gmp, args)



# SPDX-FileCopyrightText: 2024 Martin Boller
#
# SPDX-License-Identifier: GPL-3.0-or-later

from argparse import Namespace

from gvm.protocols.gmp import Gmp
from gvmtools.helper import Table


def main(gmp: Gmp, args: Namespace) -> None:
    # pylint: disable=unused-argument

    response_xml = gmp.get_roles(filter_string="rows=-1")
    roles_xml = response_xml.xpath("role")

    heading = ["#", "Name", "Id", "Members"]

    rows = []
    numberRows = 0

    print("Listing roles.\n")

    for role in roles_xml:
        # Count number of reports
        numberRows = numberRows + 1
        # Cast/convert to text to show in list
        rowNumber = str(numberRows)

        name = "".join(role.xpath("name/text()"))
        role_id = role.get("id")
        role_members = "".join(role.xpath("users/text()"))

        rows.append([rowNumber, name, role_id, role_members])

    print(Table(heading=heading, rows=rows))


if __name__ == "__gmp__":
    main(gmp, args)



# -*- coding: utf-8 -*-
# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import getpass
import os
import string
import sys
import uuid
from random import choice, randrange

from gvm.errors import GvmError
from gvm.xml import pretty_print
from lxml import etree

__all__ = ["authenticate", "pretty_print", "run_script"]


class Table:
    def __init__(self, heading=None, rows=None, divider=" | "):
        self.heading = heading or []
        self.rows = rows or []
        self.divider = divider

    def _calculate_dimensions(self):
        column_sizes = []

        for column in self.heading:
            column_sizes.append(len(column))

        for row in self.rows:
            for i, column in enumerate(row):
                dim = column_sizes[i]
                column_size = len(column)

                if dim < column_size:
                    column_sizes[i] = column_size

        return column_sizes

    def _create_column(self, column, size):
        return f'{column}{" " * (size - len(column))}'

    def _create_row(self, columns):
        return self.divider.join(columns)

    def __str__(self):
        column_sizes = self._calculate_dimensions()

        row_strings = []

        heading_columns = []
        heading_divider_columns = []

        for i, column in enumerate(self.heading):
            column_size = column_sizes[i]

            heading_columns.append(self._create_column(column, column_size))
            heading_divider_columns.append(
                self._create_column("-" * column_size, column_size)
            )

        row_strings.append(self._create_row(heading_columns))
        row_strings.append(self._create_row(heading_divider_columns))

        for row in self.rows:
            row_columns = []

            for i, column in enumerate(row):
                column_size = column_sizes[i]
                row_columns.append(self._create_column(column, column_size))

            row_strings.append(self._create_row(row_columns))

        return "\n".join(row_strings)


def yes_or_no(question: str) -> bool:
    """Asks the user to proceed or not in a gvmtools script

    Arguments:
        question (str): The condition the user should answer
    """
    reply = str(input(question + " (y/n): ")).lower().strip()
    if reply[0] == ("y"):
        return True
    if reply[0] == ("n"):
        return False
    else:
        return yes_or_no("Please enter 'y' or 'n'")


def error_and_exit(msg: str) -> None:
    """Prints an error message and quits the gvmtools script

    Arguments:
        msg (str): The error message, that will be printed
    """
    print(f"\nError: {msg}\n", file=sys.stderr)
    sys.exit(1)


def generate_random_ips(count: int):
    """Generate count random IPv4s"""
    exclude_127 = [i for i in range(1, 256)]
    exclude_127.remove(127)
    return [
        f"{choice(exclude_127)}.{randrange(0, 256)}."
        f"{randrange(0, 256)}.{randrange(1, 256)}"
        for i in range(count)
    ]


def generate_id(
    size: int = 12, chars: str = string.ascii_uppercase + string.digits
):
    """Generate a random ID"""
    return "".join(choice(chars) for _ in range(size))


def generate_uuid():
    """Generate a random new uuid"""
    return str(uuid.uuid4())


def create_xml_tree(xml_doc):
    """Creates an XML tree that can be read by an gvmtools script

    Arguments:
        xml_doc (str): Path to the xml document
    """
    try:
        xml_tree = etree.parse(xml_doc)
        xml_tree = xml_tree.getroot()
    except IOError as err:
        error_and_exit(f"Failed to read xml_file: {str(err)} (exit)")
    except etree.Error as err:
        error_and_exit(f"Failed to parse xml_file: {str(err)} (exit)")

    if len(xml_tree) == 0:
        error_and_exit("XML file is empty (exit)")

    return xml_tree


def do_not_run_as_root():
    if hasattr(os, "geteuid") and os.geteuid() == 0:
        raise RuntimeError("This tool MUST NOT be run as root user.")


def authenticate(gmp, username=None, password=None):
    """Authentication helper

    Tries to get authentication username and password from arguments and if not
    present asks the username and/or password from the terminal.

    Arguments:
        gmp: A protocol instance
        username (:obj:`str`, optional): Username to authenticate with. If None,
            username will be read from terminal.
        password (:obj:`str`, optional): Password to authenticate with. If None,
            password will be read from the terminal.

    Returns:
        tuple: (username, password) tuple

    Raises:
        GmpError: Raises GmpError if authentication fails.
    """
    if gmp.is_authenticated():
        return

    # Ask for login credentials if none are given.
    if not username:
        while username is None or len(username) == 0:
            username = input("Enter username: ")

    if not password:
        password = getpass.getpass(f"Enter password for {username}: ")

    try:
        gmp.authenticate(username, password)
        return (username, password)
    except GvmError as e:
        print("Could not authenticate. Please check your credentials.")
        raise e


def run_script(path, global_vars) -> None:
    """Loads and executes a file as a python script

    Arguments:
        path (str): Path to the script file
        vars (dict): Variables passed as globals to the script
    """
    try:
        file = open(path, "r", encoding="utf-8", newline="").read()
    except FileNotFoundError:
        print(f"Script {path} does not exist", file=sys.stderr)
        sys.exit(2)

    exec(file, global_vars)  # pylint: disable=exec-used
